@@ -7,15 +7,15 @@ module Game.LambdaHack.Atomic.HandleAtomicWrite
   ( handleUpdAtomic
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
-  , updCreateActor, updDestroyActor, updCreateItem, updDestroyItem
-  , updSpotItemBag, updLoseItemBag
+  , updRegisterItems, updCreateActor, updDestroyActor
+  , updCreateItem, updDestroyItem, updSpotItemBag, updLoseItemBag
   , updMoveActor, updWaitActor, updDisplaceActor, updMoveItem
   , updRefillHP, updRefillCalm
-  , updTrajectory, updQuitFaction, updLeadFaction
-  , updDiplFaction, updTacticFaction, updAutoFaction, updRecordKill
-  , updAlterTile, updAlterExplorable, updSearchTile, updSpotTile, updLoseTile
-  , updAlterSmell, updSpotSmell, updLoseSmell, updTimeItem
-  , updAgeGame, updUnAgeGame, ageLevel, updDiscover, updCover
+  , updTrajectory, updQuitFaction, updSpotStashFaction, updLoseStashFaction
+  , updLeadFaction, updDiplFaction, updDoctrineFaction, updAutoFaction
+  , updRecordKill, updAlterTile, updAlterExplorable, updSearchTile
+  , updSpotTile, updLoseTile, updAlterSmell, updSpotSmell, updLoseSmell
+  , updTimeItem, updAgeGame, updUnAgeGame, ageLevel, updDiscover, updCover
   , updDiscoverKind, discoverKind, updCoverKind
   , updDiscoverAspect, discoverAspect, updCoverAspect
   , updDiscoverServer, updCoverServer
@@ -28,6 +28,7 @@ import Prelude ()
 import Game.LambdaHack.Core.Prelude
 
 import qualified Data.EnumMap.Strict as EM
+import qualified Data.EnumSet as ES
 import           Data.Int (Int64)
 
 import           Game.LambdaHack.Atomic.CmdAtomic
@@ -41,17 +42,17 @@ import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.MonadStateRead
 import           Game.LambdaHack.Common.Perception
+import           Game.LambdaHack.Common.Point
+import qualified Game.LambdaHack.Common.PointArray as PointArray
 import           Game.LambdaHack.Common.State
 import qualified Game.LambdaHack.Common.Tile as Tile
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Types
 import           Game.LambdaHack.Common.Vector
+import           Game.LambdaHack.Content.FactionKind
 import           Game.LambdaHack.Content.ItemKind (ItemKind)
-import           Game.LambdaHack.Content.ModeKind
 import qualified Game.LambdaHack.Content.PlaceKind as PK
 import           Game.LambdaHack.Content.TileKind (TileKind, unknownId)
-import           Game.LambdaHack.Common.Point
-import qualified Game.LambdaHack.Common.PointArray as PointArray
 import qualified Game.LambdaHack.Definition.Ability as Ability
 import           Game.LambdaHack.Definition.Defs
 
@@ -72,16 +73,17 @@ import           Game.LambdaHack.Definition.Defs
 -- that can see it, even though not all are able to process it.
 handleUpdAtomic :: MonadStateWrite m => UpdAtomic -> m ()
 handleUpdAtomic cmd = case cmd of
+  UpdRegisterItems ais -> updRegisterItems ais
   UpdCreateActor aid body ais -> updCreateActor aid body ais
   UpdDestroyActor aid body ais -> updDestroyActor aid body ais
-  UpdCreateItem iid item kit c -> updCreateItem iid item kit c
-  UpdDestroyItem iid item kit c -> updDestroyItem iid item kit c
-  UpdSpotActor aid body ais -> updCreateActor aid body ais
-  UpdLoseActor aid body ais -> updDestroyActor aid body ais
-  UpdSpotItem _ iid item kit c -> updCreateItem iid item kit c
-  UpdLoseItem _ iid item kit c -> updDestroyItem iid item kit c
-  UpdSpotItemBag c bag ais -> updSpotItemBag c bag ais
-  UpdLoseItemBag c bag ais -> updLoseItemBag c bag ais
+  UpdCreateItem _ iid item kit c -> updCreateItem iid item kit c
+  UpdDestroyItem _ iid item kit c -> updDestroyItem iid item kit c
+  UpdSpotActor aid body -> updSpotActor aid body
+  UpdLoseActor aid body -> updLoseActor aid body
+  UpdSpotItem _ iid kit c -> updSpotItem iid kit c
+  UpdLoseItem _ iid kit c -> updLoseItem iid kit c
+  UpdSpotItemBag _ c bag -> updSpotItemBag c bag
+  UpdLoseItemBag _ c bag -> updLoseItemBag c bag
   UpdMoveActor aid fromP toP -> updMoveActor aid fromP toP
   UpdWaitActor aid fromWS toWS -> updWaitActor aid fromWS toWS
   UpdDisplaceActor source target -> updDisplaceActor source target
@@ -90,10 +92,12 @@ handleUpdAtomic cmd = case cmd of
   UpdRefillCalm aid n -> updRefillCalm aid n
   UpdTrajectory aid fromT toT -> updTrajectory aid fromT toT
   UpdQuitFaction fid fromSt toSt _ -> updQuitFaction fid fromSt toSt
+  UpdSpotStashFaction _ fid lid pos -> updSpotStashFaction fid lid pos
+  UpdLoseStashFaction _ fid lid pos -> updLoseStashFaction fid lid pos
   UpdLeadFaction fid source target -> updLeadFaction fid source target
   UpdDiplFaction fid1 fid2 fromDipl toDipl ->
     updDiplFaction fid1 fid2 fromDipl toDipl
-  UpdTacticFaction fid toT fromT -> updTacticFaction fid toT fromT
+  UpdDoctrineFaction fid toT fromT -> updDoctrineFaction fid toT fromT
   UpdAutoFaction fid st -> updAutoFaction fid st
   UpdRecordKill aid ikind k -> updRecordKill aid ikind k
   UpdAlterTile lid p fromTile toTile -> updAlterTile lid p fromTile toTile
@@ -128,12 +132,78 @@ handleUpdAtomic cmd = case cmd of
   UpdKillExit{} -> return ()
   UpdWriteSave -> return ()
   UpdHearFid{} -> return ()
+  UpdMuteMessages{} -> return ()
+
+-- Actor's items may or may not be already present in @sitemD@,
+-- regardless if they are already present otherwise in the dungeon.
+-- We re-add them all to save time determining which really need it.
+-- If collision occurs, pick the item found on easier level.
+updRegisterItems :: MonadStateWrite m => [(ItemId, Item)] -> m ()
+updRegisterItems ais = do
+  let h item1 item2 =
+        assert (itemsMatch item1 item2
+                `blame` "inconsistent added items"
+                `swith` (item1, item2, ais))
+               item2 -- keep the first found level
+  forM_ ais $ \(iid, item) -> do
+    let f = case jkind item of
+          IdentityObvious _ -> id
+          IdentityCovered ix _ ->
+            updateItemIxMap $ EM.insertWith ES.union ix (ES.singleton iid)
+    modifyState $ f . updateItemD (EM.insertWith h iid item)
 
 -- Note: after this command, usually a new leader
 -- for the party should be elected (in case this actor is the only one alive).
 updCreateActor :: MonadStateWrite m
                => ActorId -> Actor -> [(ItemId, Item)] -> m ()
 updCreateActor aid body ais = do
+  updRegisterItems ais
+  updSpotActor aid body
+
+-- If a leader dies, a new leader should be elected on the server
+-- before this command is executed (not checked).
+updDestroyActor :: MonadStateWrite m
+                => ActorId -> Actor -> [(ItemId, Item)] -> m ()
+updDestroyActor aid body ais = do
+  -- Assert that actor's items belong to @sitemD@. Do not remove those
+  -- that do not appear anywhere else, for simplicity and speed.
+  itemD <- getsState sitemD
+  let match (iid, item) = itemsMatch (itemD EM.! iid) item
+  let !_A = assert (allB match ais `blame` "destroyed actor items not found"
+                    `swith` (aid, body, ais, itemD)) ()
+  updLoseActor aid body
+
+-- Create a few copies of an item that is already registered for the dungeon
+-- (in @sitemRev@ field of @StateServer@).
+--
+-- Number of copies may be zero, when the item is only created as a sample
+-- to let the player know what can potentially be genereated in the dungeon.
+updCreateItem :: MonadStateWrite m
+              => ItemId -> Item -> ItemQuant -> Container -> m ()
+updCreateItem iid item kit c = do
+  updRegisterItems [(iid, item)]
+  updSpotItem iid kit c
+
+-- Destroy some copies (possibly not all) of an item.
+updDestroyItem :: MonadStateWrite m
+               => ItemId -> Item -> ItemQuant -> Container -> m ()
+updDestroyItem iid item kit@(k, _) c = assert (k > 0) $ do
+  -- Do not remove the item from @sitemD@ nor from @sitemRev@
+  -- nor from @DiscoveryAspect@, @ItemIxMap@, etc.
+  -- It's incredibly costly and not particularly noticeable for the player.
+  -- Moreover, copies of the item may reappear in the future
+  -- and then we save computation and the player remembers past discovery.
+  -- However, assert the item is registered in @sitemD@.
+  itemD <- getsState sitemD
+  let !_A = assert ((case iid `EM.lookup` itemD of
+                        Nothing -> False
+                        Just item0 -> itemsMatch item0 item)
+                    `blame` "item already removed"
+                    `swith` (iid, item, itemD)) ()
+  updLoseItem iid kit c
+
+updSpotActor :: MonadStateWrite m => ActorId -> Actor -> m ()
+updSpotActor aid body = do
   -- The exception is possible, e.g., when we teleport and so see our actor
   -- at the new location, but also the location is part of new perception,
   -- so @UpdSpotActor@ is sent.
@@ -155,21 +225,11 @@ updCreateActor aid body ais = do
   updateLevel (blid body) $ if bproj body
                             then updateProjMap (EM.alter g (bpos body))
                             else updateBigMap (EM.alter h (bpos body))
-  addAis ais
   actorMaxSk <- getsState $ maxSkillsFromActor body
   modifyState $ updateActorMaxSkills $ EM.insert aid actorMaxSk
 
--- If a leader dies, a new leader should be elected on the server
--- before this command is executed (not checked).
-updDestroyActor :: MonadStateWrite m
-                => ActorId -> Actor -> [(ItemId, Item)] -> m ()
-updDestroyActor aid body ais = do
-  -- Assert that actor's items belong to @sitemD@. Do not remove those
-  -- that do not appear anywhere else, for simplicity and speed.
-  itemD <- getsState sitemD
-  let match (iid, item) = itemsMatch (itemD EM.! iid) item
-  let !_A = assert (allB match ais `blame` "destroyed actor items not found"
-                    `swith` (aid, body, ais, itemD)) ()
+updLoseActor :: MonadStateWrite m => ActorId -> Actor ->  m ()
+updLoseActor aid body = do
   -- Remove actor from @sactorD@.
   let f Nothing = error $ "actor already removed" `showFailure` (aid, body)
       f (Just b) = assert (b == body `blame` "inconsistent destroyed actor body"
@@ -197,15 +257,9 @@ updDestroyActor aid body ais = do
                             else updateBigMap (EM.alter h (bpos body))
   modifyState $ updateActorMaxSkills $ EM.delete aid
 
--- Create a few copies of an item that is already registered for the dungeon
--- (in @sitemRev@ field of @StateServer@).
---
--- Number of copies may be zero, when the item is only created as a sample
--- to let the player know what can potentially be genereated in the dungeon.
-updCreateItem :: MonadStateWrite m
-              => ItemId -> Item -> ItemQuant -> Container -> m ()
-updCreateItem iid item kit@(k, _) c = do
-  addAis [(iid, item)]
+updSpotItem :: MonadStateWrite m => ItemId -> ItemQuant -> Container -> m ()
+updSpotItem iid kit@(k, _) c = do
+  item <- getsState $ getItemBody iid
   when (k > 0) $ do
     insertItemContainer iid kit c
     case c of
@@ -213,58 +267,40 @@ updCreateItem iid item kit@(k, _) c = do
                           $ addItemToActorMaxSkills iid item k aid
       _ -> return ()
 
--- Destroy some copies (possibly not all) of an item.
-updDestroyItem :: MonadStateWrite m
-               => ItemId -> Item -> ItemQuant -> Container -> m ()
-updDestroyItem iid item kit@(k, _) c = assert (k > 0) $ do
+updLoseItem :: MonadStateWrite m => ItemId -> ItemQuant -> Container -> m ()
+updLoseItem iid kit@(k, _) c = assert (k > 0) $ do
+  item <- getsState $ getItemBody iid
   deleteItemContainer iid kit c
-  -- Do not remove the item from @sitemD@ nor from @sitemRev@
-  -- nor from @DiscoveryAspect@, @ItemIxMap@, etc.
-  -- It's incredibly costly and not particularly noticeable for the player.
-  -- Moreover, copies of the item may reappear in the future
-  -- and then we save computation and the player remembers past discovery.
-  -- However, assert the item is registered in @sitemD@.
-  itemD <- getsState sitemD
-  let !_A = assert ((case iid `EM.lookup` itemD of
-                        Nothing -> False
-                        Just item0 -> itemsMatch item0 item)
-                    `blame` "item already removed"
-                    `swith` (iid, item, itemD)) ()
   case c of
     CActor aid store -> when (store `elem` [CEqp, COrgan])
                         $ addItemToActorMaxSkills iid item (-k) aid
     _ -> return ()
 
-updSpotItemBag :: MonadStateWrite m
-               => Container -> ItemBag -> [(ItemId, Item)] -> m ()
-updSpotItemBag c bag ais = do
-  addAis ais
+updSpotItemBag :: MonadStateWrite m => Container -> ItemBag -> m ()
+updSpotItemBag c bag =
   -- The case of empty bag is for a hack to help identifying sample items.
-  when (not $ EM.null bag) $ do
-    let !_A = assert (EM.size bag == length ais) ()
+  unless (EM.null bag) $ do
     insertBagContainer bag c
     case c of
       CActor aid store ->
-        when (store `elem` [CEqp, COrgan]) $
+        when (store `elem` [CEqp, COrgan]) $ do
+          itemD <- getsState sitemD
+          let ais = map (\iid -> (iid, itemD EM.! iid)) $ EM.keys bag
           forM_ ais $ \(iid, item) ->
             addItemToActorMaxSkills iid item (fst $ bag EM.! iid) aid
       _ -> return ()
 
-updLoseItemBag :: MonadStateWrite m
-               => Container -> ItemBag -> [(ItemId, Item)] -> m ()
-updLoseItemBag c bag ais = assert (EM.size bag > 0
-                                   && EM.size bag == length ais) $ do
+updLoseItemBag :: MonadStateWrite m => Container -> ItemBag -> m ()
+updLoseItemBag c bag = assert (EM.size bag > 0) $ do
   deleteBagContainer bag c
   -- Do not remove the items from @sitemD@ nor from @sitemRev@,
   -- It's incredibly costly and not noticeable for the player.
   -- However, assert the items are registered in @sitemD@.
-  itemD <- getsState sitemD
-  let match (iid, item) = itemsMatch (itemD EM.! iid) item
-  let !_A = assert (allB match ais `blame` "items already removed"
-                                   `swith` (c, bag, ais, itemD)) ()
   case c of
     CActor aid store ->
-      when (store `elem` [CEqp, COrgan]) $
+      when (store `elem` [CEqp, COrgan]) $ do
+        itemD <- getsState sitemD
+        let ais = map (\iid -> (iid, itemD EM.! iid)) $ EM.keys bag
         forM_ ais $ \(iid, item) ->
           addItemToActorMaxSkills iid item (- (fst $ bag EM.! iid)) aid
     _ -> return ()
@@ -387,6 +423,28 @@ updQuitFaction fid fromSt toSt = do
   let adj fa = fa {gquit = toSt}
   updateFaction fid adj
 
+updSpotStashFaction :: MonadStateWrite m
+                    => FactionId -> LevelId -> Point -> m ()
+updSpotStashFaction fid lid pos = do
+  let adj fa = fa {gstash = Just (lid, pos)}
+    -- the stash may be outdated, but not empty and it's correct,
+    -- because we know stash may be only one, so here it's added,
+    -- the old one is removed, despite us not seeing its location;
+    -- warning: in this form, this is not reversible, no undo,
+    -- so we'd need to add the required @UpdLoseStashFaction@
+    -- elsehwere, similarly as @LoseTile@ is added when FOV
+    -- reveals that tile is different than expected
+  updateFaction fid adj
+
+updLoseStashFaction :: MonadStateWrite m
+                    => FactionId -> LevelId -> Point -> m ()
+updLoseStashFaction fid lid pos = do
+  let adj fa = assert (gstash fa == Just (lid, pos)
+                       `blame` "unexpected lack of gstash"
+                       `swith` (fid, lid, pos, fa))
+               $ fa {gstash = Nothing}
+  updateFaction fid adj
+
 -- The previous leader is assumed to be alive.
 updLeadFaction :: MonadStateWrite m
                => FactionId
@@ -395,7 +453,7 @@ updLeadFaction :: MonadStateWrite m
                -> m ()
 updLeadFaction fid source target = assert (source /= target) $ do
   fact <- getsState $ (EM.! fid) . sfactionD
-  let !_A = assert (fleaderMode (gplayer fact) /= LeaderNull) ()
+  let !_A = assert (fhasPointman (gkind fact)) ()
     -- @PosNone@ ensures this
   mtb <- getsState $ \s -> flip getActorBody s <$> target
   let !_A = assert (maybe True (not . bproj) mtb
@@ -412,28 +470,25 @@ updDiplFaction fid1 fid2 fromDipl toDipl =
   assert (fid1 /= fid2 && fromDipl /= toDipl) $ do
     fact1 <- getsState $ (EM.! fid1) . sfactionD
     fact2 <- getsState $ (EM.! fid2) . sfactionD
-    let !_A = assert (fromDipl == EM.findWithDefault Unknown fid2 (gdipl fact1)
-                      && fromDipl == EM.findWithDefault Unknown fid1 (gdipl fact2)
-                      `blame` "unexpected actor diplomacy status"
-                      `swith` (fid1, fid2, fromDipl, toDipl, fact1, fact2)) ()
+    let !_A =
+          assert (fromDipl == EM.findWithDefault Unknown fid2 (gdipl fact1)
+                  && fromDipl == EM.findWithDefault Unknown fid1 (gdipl fact2)
+                  `blame` "unexpected actor diplomacy status"
+                  `swith` (fid1, fid2, fromDipl, toDipl, fact1, fact2)) ()
     let adj fid fact = fact {gdipl = EM.insert fid toDipl (gdipl fact)}
     updateFaction fid1 (adj fid2)
     updateFaction fid2 (adj fid1)
 
-updTacticFaction :: MonadStateWrite m
-                 => FactionId -> Ability.Tactic -> Ability.Tactic -> m ()
-updTacticFaction fid toT fromT = do
-  let adj fact =
-        let player = gplayer fact
-        in assert (ftactic player == fromT)
-           $ fact {gplayer = player {ftactic = toT}}
+updDoctrineFaction :: MonadStateWrite m
+                   => FactionId -> Ability.Doctrine -> Ability.Doctrine -> m ()
+updDoctrineFaction fid toT fromT = do
+  let adj fact = assert (gdoctrine fact == fromT) $ fact {gdoctrine = toT}
   updateFaction fid adj
 
 updAutoFaction :: MonadStateWrite m => FactionId -> Bool -> m ()
 updAutoFaction fid st =
   updateFaction fid (\fact ->
-    assert (isAIFact fact == not st)
-    $ fact {gplayer = automatePlayer st (gplayer fact)})
+    assert (gunderAI fact == not st) $ fact {gunderAI = st})
 
 -- Record a given number (usually just 1, or -1 for undo) of actor kills
 -- for score calculation.
@@ -466,14 +521,14 @@ updAlterTile lid p fromTile toTile = assert (fromTile /= toTile) $ do
   lvl <- getLevel lid
   let t = lvl `at` p
   if t /= fromTile
-  then atomicFail "tile to alter is different than assumed"
+  then atomicFail "terrain to modify is different than assumed"
   else do
     let adj ts = ts PointArray.// [(p, toTile)]
     updateLevel lid $ updateTile adj
     case ( Tile.isExplorable coTileSpeedup fromTile
          , Tile.isExplorable coTileSpeedup toTile ) of
-      (False, True) -> updateLevel lid $ \lvl2 -> lvl2 {lseen = lseen lvl + 1}
-      (True, False) -> updateLevel lid $ \lvl2 -> lvl2 {lseen = lseen lvl - 1}
+      (False, True) -> updateLevel lid $ \lvl2 -> lvl2 {lseen = lseen lvl2 + 1}
+      (True, False) -> updateLevel lid $ \lvl2 -> lvl2 {lseen = lseen lvl2 - 1}
       _ -> return ()
 
 updAlterExplorable :: MonadStateWrite m => LevelId -> Int -> m ()
@@ -507,7 +562,8 @@ updSpotTile :: MonadStateWrite m
 updSpotTile lid ts = assert (not $ null ts) $ do
   COps{coTileSpeedup} <- getsState scops
   let unk tileMap (p, _) = tileMap PointArray.! p == unknownId
-      adj tileMap = assert (all (unk tileMap) ts) $ tileMap PointArray.// ts
+      adj tileMap = assert (allB (unk tileMap) ts)
+                    $ tileMap PointArray.// ts
   updateLevel lid $ updateTile adj
   let f (_, t1) = when (Tile.isExplorable coTileSpeedup t1) $
         updateLevel lid $ \lvl -> lvl {lseen = lseen lvl + 1}
@@ -521,7 +577,8 @@ updLoseTile lid ts = assert (not $ null ts) $ do
   COps{coTileSpeedup} <- getsState scops
   let matches tileMap (p, ov) = tileMap PointArray.! p == ov
       tu = map (second (const unknownId)) ts
-      adj tileMap = assert (all (matches tileMap) ts) $ tileMap PointArray.// tu
+      adj tileMap = assert (allB (matches tileMap) ts)
+                    $ tileMap PointArray.// tu
   updateLevel lid $ updateTile adj
   let f (_, t1) = when (Tile.isExplorable coTileSpeedup t1) $
         updateLevel lid $ \lvl -> lvl {lseen = lseen lvl - 1}
@@ -576,26 +633,27 @@ updLoseSmell lid sms = assert (not $ null sms) $ do
   updateLevel lid $ updateSmell upd
 
 updTimeItem :: MonadStateWrite m
-            => ItemId -> Container -> ItemTimer -> ItemTimer
+            => ItemId -> Container -> ItemTimers -> ItemTimers
             -> m ()
 updTimeItem iid c fromIt toIt = assert (fromIt /= toIt) $ do
   bag <- getsState $ getContainerBag c
   case iid `EM.lookup` bag of
     Just (k, it) -> do
-      let !_A = assert (fromIt == it `blame` (k, it, iid, c, fromIt, toIt)) ()
+      let !_A1 = assert (fromIt == it `blame` (k, it, iid, c, fromIt, toIt)) ()
+          !_A2 = assert (length toIt <= k `blame` (k, toIt, iid, c, fromIt)) ()
       deleteItemContainer iid (k, fromIt) c
       insertItemContainer iid (k, toIt) c
     Nothing -> error $ "" `showFailure` (bag, iid, c, fromIt, toIt)
 
-updAgeGame :: MonadStateWrite m => [LevelId] -> m ()
+updAgeGame :: MonadStateWrite m => ES.EnumSet LevelId -> m ()
 updAgeGame lids = do
   modifyState $ updateTime $ flip timeShift (Delta timeClip)
-  mapM_ (ageLevel (Delta timeClip)) lids
+  mapM_ (ageLevel (Delta timeClip)) $ ES.elems lids
 
-updUnAgeGame :: MonadStateWrite m => [LevelId] -> m ()
+updUnAgeGame :: MonadStateWrite m => ES.EnumSet LevelId -> m ()
 updUnAgeGame lids = do
   modifyState $ updateTime $ flip timeShift (timeDeltaReverse $ Delta timeClip)
-  mapM_ (ageLevel (timeDeltaReverse $ Delta timeClip)) lids
+  mapM_ (ageLevel (timeDeltaReverse $ Delta timeClip)) $ ES.elems lids
 
 ageLevel :: MonadStateWrite m => Delta Time -> LevelId -> m ()
 ageLevel delta lid =
@@ -688,11 +746,14 @@ updCoverServer iid arItem =
     assert (discoAspect1 EM.! iid == arItem)
     $ EM.delete iid discoAspect1
 
+-- This is ever run only on clients.
 updRestart :: MonadStateWrite m => State -> m ()
 updRestart = putState
 
+-- This is ever run only on the server.
 updRestartServer :: MonadStateWrite m => State -> m ()
 updRestartServer = putState
 
+-- This is ever run only on the server.
 updResumeServer :: MonadStateWrite m => State -> m ()
 updResumeServer = putState
