@@ -1,13 +1,11 @@
 -- | Assorted conditions used later on in AI logic.
 module Game.LambdaHack.Client.AI.ConditionM
-  ( condAimEnemyPresentM
-  , condAimEnemyRememberedM
+  ( condAimEnemyTargetedM
+  , condAimEnemyOrStashM
+  , condAimEnemyOrRememberedM
   , condAimNonEnemyPresentM
-  , condAimEnemyNoMeleeM
-  , condInMeleeM
   , condAimCrucialM
   , condTgtNonmovingEnemyM
-  , condAnyFoeAdjM
   , condAdjTriggerableM
   , meleeThreatDistList
   , condBlocksFriendsM
@@ -21,7 +19,7 @@ module Game.LambdaHack.Client.AI.ConditionM
   , benGroundItems
   , desirableItem
   , condSupport
-  , condSoloM
+  , condAloneM
   , condShineWouldBetrayM
   , fleeList
   ) where
@@ -31,10 +29,8 @@ import Prelude ()
 import Game.LambdaHack.Core.Prelude
 
 import qualified Data.EnumMap.Strict as EM
-import           Data.Ord
 
 import           Game.LambdaHack.Client.Bfs
-import           Game.LambdaHack.Client.CommonM
 import           Game.LambdaHack.Client.MonadClient
 import           Game.LambdaHack.Client.State
 import           Game.LambdaHack.Common.Actor
@@ -45,74 +41,62 @@ import qualified Game.LambdaHack.Common.ItemAspect as IA
 import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.MonadStateRead
+import           Game.LambdaHack.Common.Point
 import           Game.LambdaHack.Common.ReqFailure
 import           Game.LambdaHack.Common.State
 import qualified Game.LambdaHack.Common.Tile as Tile
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Types
 import           Game.LambdaHack.Common.Vector
+import           Game.LambdaHack.Content.FactionKind
 import qualified Game.LambdaHack.Content.ItemKind as IK
-import           Game.LambdaHack.Content.ModeKind
-import           Game.LambdaHack.Content.RuleKind
+import qualified Game.LambdaHack.Content.RuleKind as RK
 import qualified Game.LambdaHack.Core.Dice as Dice
-import           Game.LambdaHack.Common.Point
 import qualified Game.LambdaHack.Definition.Ability as Ability
 import           Game.LambdaHack.Definition.Defs
 
 -- All conditions are (partially) lazy, because they are not always
 -- used in the strict monadic computations they are in.
 
--- | Require that the target enemy is visible by the party.
-condAimEnemyPresentM :: MonadClient m => ActorId -> m Bool
-condAimEnemyPresentM aid = do
+-- | Require that a target enemy is visible by the party.
+condAimEnemyTargetedM :: MonadClientRead m => ActorId -> m Bool
+condAimEnemyTargetedM aid = do
   btarget <- getsClient $ getTarget aid
   return $ case btarget of
     Just (TEnemy _) -> True
     _ -> False
 
--- | Require that the target enemy is remembered on the actor's level.
-condAimEnemyRememberedM :: MonadClient m => ActorId -> m Bool
-condAimEnemyRememberedM aid = do
+-- | Require that a target enemy or enemy stash is visible by the party.
+condAimEnemyOrStashM :: MonadClientRead m => ActorId -> m Bool
+condAimEnemyOrStashM aid = do
+  btarget <- getsClient $ getTarget aid
+  return $ case btarget of
+    Just (TEnemy _) -> True
+    Just (TPoint (TStash _) _ _) -> True  -- speedup from: lid == blid b
+    _ -> False
+
+-- | Require that a target enemy is remembered on the actor's level.
+condAimEnemyOrRememberedM :: MonadClientRead m => ActorId -> m Bool
+condAimEnemyOrRememberedM aid = do
   b <- getsState $ getActorBody aid
   btarget <- getsClient $ getTarget aid
   return $ case btarget of
+    Just (TEnemy _) -> True
     Just (TPoint (TEnemyPos _) lid _) -> lid == blid b
+    Just (TPoint (TStash _) lid _) -> lid == blid b
     _ -> False
 
--- | Require that the target non-enemy is visible by the party.
-condAimNonEnemyPresentM :: MonadClient m => ActorId -> m Bool
+-- | Require that a target non-enemy is visible by the party.
+condAimNonEnemyPresentM :: MonadClientRead m => ActorId -> m Bool
 condAimNonEnemyPresentM aid = do
   btarget <- getsClient $ getTarget aid
   return $ case btarget of
     Just (TNonEnemy _) -> True
     _ -> False
 
--- | Require that the target enemy is visible by the party and doesn't melee.
-condAimEnemyNoMeleeM :: MonadClient m => ActorId -> m Bool
-condAimEnemyNoMeleeM aid = do
-  btarget <- getsClient $ getTarget aid
-  case btarget of
-    Just (TEnemy aid2) -> do
-      b2 <- getsState $ getActorBody aid2
-      actorMaxSkills <- getsState sactorMaxSkills
-      return $ actorCanMelee actorMaxSkills aid2 b2
-    _ -> return False
-
-condInMeleeM :: MonadClient m => LevelId -> m Bool
-condInMeleeM lid = do
-  condInMelee <- getsClient scondInMelee
-  case EM.lookup lid condInMelee of
-    Just inM -> return inM
-    Nothing -> do
-      side <- getsClient sside
-      inM <- getsState $ inMelee side lid
-      modifyClient $ \cli ->
-        cli {scondInMelee = EM.insert lid inM condInMelee}
-      return inM
-
 -- | Require that the target is crucial to success, e.g., an item,
 -- or that it's not too far away and so the changes to get it are high.
-condAimCrucialM :: MonadClient m => ActorId -> m Bool
+condAimCrucialM :: MonadClientRead m => ActorId -> m Bool
 condAimCrucialM aid = do
   b <- getsState $ getActorBody aid
   mtgtMPath <- getsClient $ EM.lookup aid . stargetD
@@ -123,13 +107,14 @@ condAimCrucialM aid = do
       && (pathLen < 10  -- close enough to get there first
           || tgoal `notElem` [TUnknown, TKnown])
     Just TgtAndPath{tapTgt=TVector{}, tapPath=Just AndPath{pathLen}} ->
-      pathLen < 7  -- the constant in @vToTgt@, where only
-                   -- non-crucial targets are produced; this will also
-                   -- prevent animals from sleep close to cave edges
+      pathLen < 7  -- can't say if the target important, but the constants
+                   -- from @take6@ and @traSlack7@ ensure target is
+                   -- already approached or close to level edge
+                   -- or not a random @traSlack7@ wandering
     _ -> False  -- includes the case of target with no path
 
 -- | Check if the target is a nonmoving enemy.
-condTgtNonmovingEnemyM :: MonadClient m => ActorId -> m Bool
+condTgtNonmovingEnemyM :: MonadClientRead m => ActorId -> m Bool
 condTgtNonmovingEnemyM aid = do
   btarget <- getsClient $ getTarget aid
   case btarget of
@@ -138,19 +123,13 @@ condTgtNonmovingEnemyM aid = do
       return $ Ability.getSk Ability.SkMove actorMaxSk <= 0
     _ -> return False
 
--- | Require that any non-dying foe is adjacent, except projectiles
--- that (possibly) explode upon contact.
-condAnyFoeAdjM :: MonadStateRead m => ActorId -> m Bool
-condAnyFoeAdjM aid = getsState $ anyFoeAdj aid
-
 -- | Require the actor stands on or adjacent to a triggerable tile
 -- (e.g., stairs).
-condAdjTriggerableM :: MonadClient m => ActorId -> m Bool
-condAdjTriggerableM aid = do
+condAdjTriggerableM :: MonadStateRead m => Ability.Skills -> ActorId -> m Bool
+condAdjTriggerableM actorSk aid = do
   COps{coTileSpeedup} <- getsState scops
   b <- getsState $ getActorBody aid
   lvl <- getLevel $ blid b
-  actorSk <- currentSkillsClient aid
   let alterSkill = Ability.getSk Ability.SkAlter actorSk
       alterMinSkill p = Tile.alterMinSkill coTileSpeedup $ lvl `at` p
       underFeet p = p == bpos b  -- if enter and alter, be more permissive
@@ -169,22 +148,23 @@ condAdjTriggerableM aid = do
 -- because they can't chase us and also because they can't be aggresive
 -- so to resolve the stalemate, the opposing AI has to be aggresive
 -- by ignoring them and closing in to melee distance.
-meleeThreatDistList :: ActorId -> State -> [(Int, (ActorId, Actor))]
-meleeThreatDistList aid s =
+meleeThreatDistList :: [(ActorId, Actor)] -> ActorId -> State
+                    -> [(Int, (ActorId, Actor))]
+meleeThreatDistList foeAssocs aid s =
   let actorMaxSkills = sactorMaxSkills s
       b = getActorBody aid s
-      allAtWar = foeRegularAssocs (bfid b) (blid b) s
+
       strongActor (aid2, b2) =
         let actorMaxSk = actorMaxSkills EM.! aid2
             nonmoving = Ability.getSk Ability.SkMove actorMaxSk <= 0
         in not (hpTooLow b2 actorMaxSk || nonmoving)
-           && actorCanMelee actorMaxSkills aid2 b2
-      allThreats = filter strongActor allAtWar
+           && actorCanMeleeToHarm actorMaxSkills aid2 b2
+      allThreats = filter strongActor foeAssocs
       addDist (aid2, b2) = (chessDist (bpos b) (bpos b2), (aid2, b2))
   in sortBy (comparing fst) $ map addDist allThreats
 
 -- | Require the actor blocks the paths of any of his party members.
-condBlocksFriendsM :: MonadClient m => ActorId -> m Bool
+condBlocksFriendsM :: MonadClientRead m => ActorId -> m Bool
 condBlocksFriendsM aid = do
   b <- getsState $ getActorBody aid
   targetD <- getsClient stargetD
@@ -195,7 +175,8 @@ condBlocksFriendsM aid = do
           _ -> False
   any blocked <$> getsState (fidActorRegularIds (bfid b) (blid b))
 
--- | Require the actor stands over a weapon that would be auto-equipped.
+-- | Require the actor stands over a weapon that would be auto-equipped,
+-- if only it was a desirable item (checked elsewhere).
 condFloorWeaponM :: MonadStateRead m => ActorId -> m Bool
 condFloorWeaponM aid =
   any (IA.checkFlag Ability.Meleeable . aspectRecordFull . snd) <$>
@@ -204,51 +185,68 @@ condFloorWeaponM aid =
 -- | Check whether the actor has no weapon in equipment.
 condNoEqpWeaponM :: MonadStateRead m => ActorId -> m Bool
 condNoEqpWeaponM aid =
-  all (not . IA.checkFlag Ability.Meleeable . aspectRecordFull . snd) <$>
+  not . any (IA.checkFlag Ability.Meleeable . aspectRecordFull . snd) <$>
     getsState (fullAssocs aid [CEqp])
 
 -- | Require that the actor can project any items.
-condCanProjectM :: MonadClient m => Int -> ActorId -> m Bool
-condCanProjectM skill aid =
-  if skill < 1 then return False else  -- shortcut
+condCanProjectM :: MonadClientRead m => Int -> ActorId -> m Bool
+condCanProjectM skill aid = do
+  side <- getsClient sside
+  curChal <- getsClient scurChal
+  fact <- getsState $ (EM.! side) . sfactionD
+  if skill < 1
+     || ckeeper curChal && fhasUI (gkind fact)
+  then return False
+  else  -- shortcut
     -- Compared to conditions in @projectItem@, range and charge are ignored,
     -- because they may change by the time the position for the fling
     -- is reached.
     not . null <$> condProjectListM skill aid
 
-condProjectListM :: MonadClient m
+condProjectListM :: MonadClientRead m
                  => Int -> ActorId
-                 -> m [(Benefit, CStore, ItemId, ItemFull, ItemQuant)]
+                 -> m [(Double, CStore, ItemId, ItemFull, ItemQuant)]
 condProjectListM skill aid = do
   condShineWouldBetray <- condShineWouldBetrayM aid
-  condAimEnemyPresent <- condAimEnemyPresentM aid
+  condAimEnemyOrRemembered <- condAimEnemyOrRememberedM aid
   discoBenefit <- getsClient sdiscoBenefit
   getsState $ projectList discoBenefit skill aid
-                          condShineWouldBetray condAimEnemyPresent
+                          condShineWouldBetray condAimEnemyOrRemembered
 
 projectList :: DiscoveryBenefit -> Int -> ActorId -> Bool -> Bool -> State
-            -> [(Benefit, CStore, ItemId, ItemFull, ItemQuant)]
+            -> [(Double, CStore, ItemId, ItemFull, ItemQuant)]
 projectList discoBenefit skill aid
-            condShineWouldBetray condAimEnemyPresent s =
+            condShineWouldBetray condAimEnemyOrRemembered s =
   let b = getActorBody aid s
       actorMaxSk = getActorMaxSkills aid s
       calmE = calmEnough b actorMaxSk
-      condNotCalmEnough = not calmE
       heavilyDistressed =  -- Actor hit by a projectile or similarly distressed.
         deltasSerious (bcalmDelta b)
+      uneasy = condAimEnemyOrRemembered
+               || not calmE
+               || heavilyDistressed
+        -- don't take recent fleeing into account when item can be lost
+      coeff CGround = 2  -- pickup turn saved
+      coeff COrgan = error $ "" `showFailure` benList
+      coeff CEqp = 1000  -- must hinder currently (or be very potent);
+                         -- note: not larger, to avoid Int32 overflow
+      coeff CStash = 1
       -- This detects if the value of keeping the item in eqp is in fact < 0.
-      hind = hinders condShineWouldBetray condAimEnemyPresent
-                     heavilyDistressed condNotCalmEnough actorMaxSk
-      q (Benefit{benInEqp, benFling}, _, _, itemFull, _) =
+      hind = hinders condShineWouldBetray uneasy actorMaxSk
+      goodMissile (Benefit{benInEqp, benFling}, cstore, iid, itemFull, kit) =
         let arItem = aspectRecordFull itemFull
-        in benFling < 0
-           && (not benInEqp  -- can't wear, so OK to risk losing or breaking
-               || not (IA.checkFlag Ability.Meleeable arItem)
-                    -- anything else expendable
-                  && hind itemFull)  -- hinders now, so possibly often, so away!
-           && permittedProjectAI skill calmE itemFull
-      stores = [CEqp, CInv, CGround] ++ [CSha | calmE]
-  in filter q $ benAvailableItems discoBenefit aid stores s
+            benR = coeff cstore * benFling
+        in if benR < -1  -- ignore very weak projectiles
+              && (not benInEqp  -- can't wear, so OK to risk losing or breaking
+                  || not (IA.checkFlag Ability.Meleeable arItem)
+                       -- anything else expendable
+                     && hind itemFull)  -- hinders now, so possibly often
+              && permittedProjectAI skill calmE itemFull
+           then Just (benR, cstore, iid, itemFull, kit)
+           else Nothing
+      stores = [CStash, CGround] ++ [CEqp | calmE]
+      benList = benAvailableItems discoBenefit aid stores s
+  in mapMaybe goodMissile benList
 
 -- | Produce the list of items from the given stores available to the actor
 -- and the items' values.
@@ -256,27 +254,24 @@ benAvailableItems :: DiscoveryBenefit -> ActorId -> [CStore] -> State
                   -> [(Benefit, CStore, ItemId, ItemFull, ItemQuant)]
 benAvailableItems discoBenefit aid cstores s =
   let b = getActorBody aid s
-      ben cstore bag =
+      mstash = gstash $ sfactionD s EM.! bfid b
+      ben _ CGround | mstash == Just (blid b, bpos b) = []
+      ben bag cstore =
         [ (discoBenefit EM.! iid, cstore, iid, itemToFull iid s, kit)
         | (iid, kit) <- EM.assocs bag]
-      benCStore cs = ben cs $ getBodyStoreBag b cs s
+      benCStore cs = ben (getBodyStoreBag b cs s) cs
   in concatMap benCStore cstores
 
-hinders :: Bool -> Bool -> Bool -> Bool -> Ability.Skills -> ItemFull
-        -> Bool
-hinders condShineWouldBetray condAimEnemyPresent
-        heavilyDistressed condNotCalmEnough
-          -- guess that enemies have projectiles and used them now or recently
-        actorMaxSk itemFull =
+hinders :: Bool -> Bool -> Ability.Skills -> ItemFull -> Bool
+hinders condShineWouldBetray uneasy actorMaxSk itemFull =
   let arItem = aspectRecordFull itemFull
       itemShine = 0 < IA.getSkill Ability.SkShine arItem
       -- @condAnyFoeAdj@ is not checked, because it's transient and also item
       -- management is unlikely to happen during melee, anyway
       itemShineBad = condShineWouldBetray && itemShine
-  in -- In the presence of enemies (seen, or unseen but distressing)
+  in -- In the presence of enemies (seen, remembered or unseen but distressing)
      -- actors want to hide in the dark.
-     (condAimEnemyPresent || condNotCalmEnough || heavilyDistressed)
-     && itemShineBad  -- even if it's a weapon, take it off
+     uneasy && itemShineBad  -- even if it's a weapon, take it off
      -- Fast actors want to hit hard, because they hit much more often
      -- than receive hits.
      || gearSpeed actorMaxSk > speedWalk
@@ -285,12 +280,12 @@ hinders condShineWouldBetray condAimEnemyPresent
         && 0 > IA.getSkill Ability.SkHurtMelee arItem
 
 -- | Require that the actor stands over a desirable item.
-condDesirableFloorItemM :: MonadClient m => ActorId -> m Bool
+condDesirableFloorItemM :: MonadClientRead m => ActorId -> m Bool
 condDesirableFloorItemM aid = not . null <$> benGroundItems aid
 
 -- | Produce the list of items on the ground beneath the actor
 -- that are worth picking up.
-benGroundItems :: MonadClient m
+benGroundItems :: MonadClientRead m
                => ActorId
                -> m [(Benefit, CStore, ItemId, ItemFull, ItemQuant)]
 benGroundItems aid = do
@@ -298,22 +293,23 @@ benGroundItems aid = do
   b <- getsState $ getActorBody aid
   fact <- getsState $ (EM.! bfid b) . sfactionD
   discoBenefit <- getsClient sdiscoBenefit
-  let canEsc = fcanEscape (gplayer fact)
+  let canEsc = fcanEscape (gkind fact)
       isDesirable (ben, _, _, itemFull, _) =
         desirableItem cops canEsc (benPickup ben)
                       (aspectRecordFull itemFull) (itemKind itemFull)
-                      99  -- fake, becuase no time is wasted walking to item
+                      99  -- fake, because no time is wasted walking to item
   filter isDesirable
     <$> getsState (benAvailableItems discoBenefit aid [CGround])
 
 desirableItem :: COps -> Bool -> Double -> IA.AspectRecord -> IK.ItemKind -> Int
               -> Bool
-desirableItem COps{corule=RuleContent{rsymbolProjectile}}
+desirableItem COps{corule}
               canEsc benPickup arItem itemKind k =
-  let loneProjectile = IK.isymbol itemKind == rsymbolProjectile
-                       && k == 1
-                       && Dice.infDice (IK.icount itemKind) > 1
-                            -- never generated as lone; usually means weak
+  let loneProjectile =
+        IK.isymbol itemKind == IK.rsymbolProjectile (RK.ritemSymbols corule)
+        && k == 1
+        && Dice.infDice (IK.icount itemKind) > 1
+             -- never generated as lone; usually means weak
       useful = if canEsc
                then benPickup > 0
                     || IA.checkFlag Ability.Precious arItem
@@ -323,42 +319,45 @@ desirableItem COps{corule=RuleContent{rsymbolProjectile}}
                  in benPickup > 0 && not preciousNotUseful
   in useful && not loneProjectile
 
-condSupport :: MonadClient m => Int -> ActorId -> m Bool
+condSupport :: MonadClientRead m
+            => [(ActorId, Actor)] -> Int -> ActorId -> m Bool
 {-# INLINE condSupport #-}
-condSupport param aid = do
-  btarget <- getsClient $ getTarget aid
-  condAimEnemyPresent <- condAimEnemyPresentM aid
-  condAimEnemyRemembered <- condAimEnemyRememberedM aid
-  getsState $ strongSupport param aid btarget
-                            condAimEnemyPresent condAimEnemyRemembered
+condSupport friendAssocs param aid = do
+  mtgtMPath <- getsClient $ EM.lookup aid . stargetD
+  getsState $ strongSupport friendAssocs param aid mtgtMPath
 
-strongSupport :: Int -> ActorId -> Maybe Target -> Bool -> Bool -> State -> Bool
-strongSupport param aid btarget condAimEnemyPresent condAimEnemyRemembered s =
+strongSupport :: [(ActorId, Actor)]
+              -> Int -> ActorId -> Maybe TgtAndPath -> State
+              -> Bool
+strongSupport friendAssocs param aid mtgtMPath s =
   -- The smaller the area scanned for friends, the lower number required.
   let actorMaxSkills = sactorMaxSkills s
       actorMaxSk = actorMaxSkills EM.! aid
       n = min 2 param - Ability.getSk Ability.SkAggression actorMaxSk
       b = getActorBody aid s
-      mtgtPos = aidTgtToPos aid (blid b) btarget s
-      approaching b2 = case mtgtPos of
-        Just tgtPos | condAimEnemyPresent || condAimEnemyRemembered ->
-          chessDist (bpos b2) tgtPos <= 1 + param
+      approaching b2 = case mtgtMPath of
+        Just TgtAndPath{tapTgt=TEnemy{},tapPath=Just AndPath{pathGoal}} ->
+            chessDist (bpos b2) pathGoal <= 1 + param  -- will soon melee anyway
         _ -> False
       closeEnough b2 = let dist = chessDist (bpos b) (bpos b2)
-                       in dist > 0 && (dist <= param || approaching b2)
+                       in dist > 0 && (dist <= max 2 param || approaching b2)
       closeAndStrong (aid2, b2) = closeEnough b2
-                                  && actorCanMelee actorMaxSkills aid2 b2
-      friends = friendRegularAssocs (bfid b) (blid b) s
-      closeAndStrongFriends = filter closeAndStrong friends
+                                  && actorCanMeleeToHarm actorMaxSkills aid2 b2
+      closeAndStrongFriends = filter closeAndStrong friendAssocs
   in n <= 0 || not (null (drop (n - 1) closeAndStrongFriends))
        -- optimized: length closeAndStrongFriends >= n
 
-condSoloM :: MonadClient m => ActorId -> m Bool
-condSoloM aid = do
+-- The numbers reflect fleeing AI conditions for non-aggresive actors
+-- so that actors don't wait for support that is not possible due to not
+-- enough friends on the level, even counting sleeping ones.
+condAloneM :: MonadStateRead m => [(ActorId, Actor)] -> ActorId -> m Bool
+condAloneM friendAssocs aid = do
   b <- getsState $ getActorBody aid
-  let isSingleton [_] = True
-      isSingleton _ = False
-  isSingleton <$> getsState (friendRegularList (bfid b) (blid b))
+  mstash <- getsState $ \s -> gstash $ sfactionD s EM.! bfid b
+  let onStashLevel = case mstash of
+        Nothing -> False
+        Just (lid, _) -> lid == blid b
+  return $! length friendAssocs <= if onStashLevel then 3 else 2
 
 -- | Require that the actor stands in the dark and so would be betrayed
 -- by his own equipped light,
@@ -369,8 +368,9 @@ condShineWouldBetrayM aid = do
   return $ not aInAmbient  -- tile is dark, so actor could hide
 
 -- | Produce a list of acceptable adjacent points to flee to.
-fleeList :: MonadClient m => ActorId -> m ([(Int, Point)], [(Int, Point)])
-fleeList aid = do
+fleeList :: MonadClientRead m
+         => [(ActorId, Actor)] -> ActorId -> m ([(Int, Point)], [(Int, Point)])
+fleeList foeAssocs aid = do
   COps{coTileSpeedup} <- getsState scops
   mtgtMPath <- getsClient $ EM.lookup aid . stargetD
   -- Prefer fleeing along the path to target, unless the target is a foe,
@@ -380,19 +380,21 @@ fleeList aid = do
                        , tapTgt } -> case tapTgt of
           TEnemy{} -> Left pathGoal
           TPoint TEnemyPos{} _ _ -> Left pathGoal
+            -- this is too weak, because only one is recorded and sometimes
+            -- many are needed to decide to flee next turn as well
           _ -> Right pathList
         _ -> Right []
-  fleeD <- getsClient sfleeD
-  -- But if fled previous turn, prefer even more fleeing further this turn.
-  let eOldFleeOrTgt = case EM.lookup aid fleeD of
-        Nothing -> etgtPath
-        Just p -> Left p
   b <- getsState $ getActorBody aid
   lvl <- getLevel $ blid b
-  posFoes <- getsState $ map bpos . foeRegularList (bfid b) (blid b)
-  let myVic = vicinityUnsafe $ bpos b
-      dist p | null posFoes = 100
-             | otherwise = minimum $ map (chessDist p) posFoes
+  localTime <- getsState $ getLocalTime (blid b)
+  fleeD <- getsClient sfleeD
+  -- But if fled recently, prefer even more fleeing further this turn.
+  let eOldFleeOrTgt = case EM.lookup aid fleeD of
+        Just (fleeStart, time) | timeRecent5 localTime time -> Left fleeStart
+        _ -> etgtPath
+      myVic = vicinityUnsafe $ bpos b
+      dist p | null foeAssocs = 100
+             | otherwise = minimum $ map (chessDist p . bpos . snd) foeAssocs
       dVic = map (dist &&& id) myVic
       -- Flee, if possible. Direct access required; not enough time to open.
       -- Can't be occupied.
