@@ -1,6 +1,6 @@
 -- | Picking the AI actor to move and refreshing leader and non-leader targets.
 module Game.LambdaHack.Client.AI.PickActorM
-  ( pickActorToMove, setTargetFromTactics
+  ( pickActorToMove, setTargetFromDoctrines
   ) where
 
 import Prelude ()
@@ -19,38 +19,52 @@ import           Game.LambdaHack.Client.State
 import           Game.LambdaHack.Common.Actor
 import           Game.LambdaHack.Common.ActorState
 import           Game.LambdaHack.Common.Faction
+import           Game.LambdaHack.Common.Kind
+import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.MonadStateRead
 import           Game.LambdaHack.Common.Point
 import           Game.LambdaHack.Common.State
+import qualified Game.LambdaHack.Common.Tile as Tile
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Types
-import           Game.LambdaHack.Content.ModeKind
+import           Game.LambdaHack.Content.FactionKind (fskillsOther)
 import           Game.LambdaHack.Core.Frequency
 import           Game.LambdaHack.Core.Random
 import qualified Game.LambdaHack.Definition.Ability as Ability
 
 -- | Pick a new leader from among the actors on the current level.
 -- Refresh the target of the new leader, even if unchanged.
-pickActorToMove :: MonadClient m => Maybe ActorId -> m ActorId
-{-# INLINE pickActorToMove #-}
-pickActorToMove maidToAvoid = do
+pickActorToMove :: MonadClient m
+                => [(ActorId, Actor)] -> [(ActorId, Actor)] -> Maybe ActorId
+                -> m ActorId
+pickActorToMove foeAssocs friendAssocs maidToAvoid = do
+  COps{coTileSpeedup} <- getsState scops
   actorMaxSkills <- getsState sactorMaxSkills
   mleader <- getsClient sleader
   let oldAid = fromMaybe (error $ "" `showFailure` maidToAvoid) mleader
   oldBody <- getsState $ getActorBody oldAid
   let side = bfid oldBody
       arena = blid oldBody
+  lvl <- getLevel arena
+  localTime <- getsState $ getLocalTime arena
+  condInMelee <- condInMeleeM arena
   fact <- getsState $ (EM.! side) . sfactionD
   -- Find our actors on the current level only.
   ours <- getsState $ fidActorRegularAssocs side arena
   let pickOld = do
-        void $ refreshTarget (oldAid, oldBody)
+        void $ refreshTarget foeAssocs friendAssocs (oldAid, oldBody)
         return oldAid
       oursNotSleeping = filter (\(_, b) -> bwatch b /= WSleep) ours
+      -- Faction discourages client leader change on level, because
+      -- non-leader actors have the same skills as leader, so no point.
+      -- Server is guaranteed to switch leader within a level occasionally,
+      -- e.g., when the old leader dies, so this works fine.
+      discouragedPointmanSwitchOnLevel =
+        fskillsOther (gkind fact) == Ability.zeroSkills
   case oursNotSleeping of
-    _ | -- Keep the leader: faction discourages client leader change on level,
-        -- so will only be changed if waits (maidToAvoid)
+    _ | -- Keep the leader: client is discouraged from leader switching,
+        -- so it will only be changed if pointman waits (maidToAvoid)
         -- to avoid wasting his higher mobility.
         -- This is OK for monsters even if in melee, because both having
         -- a meleeing actor a leader (and higher DPS) and rescuing actor
@@ -58,25 +72,28 @@ pickActorToMove maidToAvoid = do
         -- And we are guaranteed that only the two classes of actors are
         -- not waiting, with some exceptions (urgent unequip, flee via starts,
         -- melee-less trying to flee, first aid, etc.).
-        snd (autoDungeonLevel fact) && isNothing maidToAvoid -> pickOld
+       discouragedPointmanSwitchOnLevel && isNothing maidToAvoid -> pickOld
     [] -> pickOld
     [(aidNotSleeping, bNotSleeping)] -> do
       -- Target of asleep actors won't change unless foe adjacent,
       -- which is caught without recourse to targeting.
-      void $ refreshTarget (aidNotSleeping, bNotSleeping)
+      void $ refreshTarget foeAssocs friendAssocs (aidNotSleeping, bNotSleeping)
       return aidNotSleeping
     _ -> do
       -- At this point we almost forget who the old leader was
       -- and treat all party actors the same, eliminating candidates
-      -- until we can't distinguish them any more, at which point we prefer
-      -- the old leader, if he is among the best candidates
+      -- until we can't distinguish them any more, at which point we slightly
+      -- prefer the old leader, if he is among the best candidates
       -- (to make the AI appear more human-like and easier to observe).
       let refresh aidBody = do
-            mtgt <- refreshTarget aidBody
+            mtgt <- refreshTarget foeAssocs friendAssocs aidBody
             return (aidBody, mtgt)
       oursTgtRaw <- mapM refresh oursNotSleeping
-      fleeD <- getsClient sfleeD
-      let goodGeneric (_, Nothing) = Nothing
+      oldFleeD <- getsClient sfleeD
+      let recentlyFled aid = maybe False
+                                   (\(_, time) -> timeRecent5 localTime time)
+                                   (aid `EM.lookup` oldFleeD)
+          goodGeneric (_, Nothing) = Nothing
           goodGeneric (_, Just TgtAndPath{tapPath=Nothing}) = Nothing
             -- this case means melee-less heroes adjacent to foes, etc.
             -- will never flee if melee is happening; but this is rare;
@@ -94,70 +111,66 @@ pickActorToMove maidToAvoid = do
             -- but it's also a rather fun exploit and a straightforward
             -- consequence of the game mechanics, so it's OK for now
           goodGeneric ((aid, b), Just tgt) = case maidToAvoid of
-            Nothing | not (aid == oldAid && actorWaits b) ->
-              -- Not the old leader that was stuck last turn
-              -- because he is likely to be still stuck.
-              Just ((aid, b), tgt)
-            Just aidToAvoid | aid /= aidToAvoid ->
-              -- Not an attempted leader stuck this turn.
-              Just ((aid, b), tgt)
-            _ -> Nothing
+            _ | aid == oldAid && actorWaits b -> Nothing
+                  -- Not the old leader that was stuck last turn
+                  -- because he is likely to be still stuck.
+            Nothing -> Just ((aid, b), tgt)
+            Just aidToAvoid ->
+              if aid == aidToAvoid
+              then Nothing  -- not an attempted leader stuck this turn
+              else Just ((aid, b), tgt)
           oursTgt = mapMaybe goodGeneric oursTgtRaw
-          -- This should be kept in sync with @actionStrategy@.
+          -- This should be kept in sync with @actionStrategy@,
+          -- because it's a part of the condition for @flee@ in @PickActionM@.
+          -- Comments are in the full copy.
           actorVulnerable ((aid, body), _) = do
-            condInMelee <- condInMeleeM $ blid body
             let actorMaxSk = actorMaxSkills EM.! aid
-            threatDistL <- getsState $ meleeThreatDistList aid
-            (fleeL, _) <- fleeList aid
-            condSupport1 <- condSupport 1 aid
-            condSupport3 <- condSupport 3 aid
-            condSolo <- condSoloM aid  -- solo fighters aggresive
-            canDeAmbientL <- getsState $ canDeAmbientList body
+            condAnyHarmfulFoeAdj <-
+              getsState $ anyHarmfulFoeAdj actorMaxSkills aid
+            threatDistL <- getsState $ meleeThreatDistList foeAssocs aid
+            (fleeL, _) <- fleeList foeAssocs aid
+            condSupport1 <- condSupport friendAssocs 1 aid
+            condSolo <- condAloneM friendAssocs aid
             let condCanFlee = not (null fleeL)
+                heavilyDistressed =
+                  deltasSerious (bcalmDelta body)
                 speed1_5 = speedScale (3%2) (gearSpeed actorMaxSk)
                 condCanMelee = actorCanMelee actorMaxSkills aid body
-                condThreat n = not $ null $ takeWhile ((<= n) . fst) threatDistL
                 threatAdj = takeWhile ((== 1) . fst) threatDistL
                 condManyThreatAdj = length threatAdj >= 2
                 condFastThreatAdj =
                   any (\(_, (aid2, _)) ->
-                    let actorMaxSk2 = actorMaxSkills EM.! aid2
-                    in gearSpeed actorMaxSk2 > speed1_5)
-                  threatAdj
-                heavilyDistressed =
-                  -- Actor hit by a projectile or similarly distressed.
-                  deltasSerious (bcalmDelta body)
-                actorShines = Ability.getSk Ability.SkShine actorMaxSk > 0
-                aCanDeLightL | actorShines = []
-                             | otherwise = canDeAmbientL
-                canFleeFromLight =
-                  not $ null $ aCanDeLightL `intersect` map snd fleeL
+                        let ar2 = actorMaxSkills EM.! aid2
+                        in gearSpeed ar2 > speed1_5)
+                      threatAdj
+                condNonStealthyThreatAdj =
+                  any (\(_, (aid2, b2)) ->
+                        let ar2 = actorMaxSkills EM.! aid2
+                        in Ability.getSk Ability.SkShine ar2 > 0
+                           || isLit (bpos b2))
+                      threatAdj
+                isLit pos = Tile.isLit coTileSpeedup (lvl `at` pos)
+                fleeingMakesSense =
+                  not condCanMelee
+                  || (Ability.getSk Ability.SkSight actorMaxSk > 2
+                      || Ability.getSk Ability.SkNocto actorMaxSk > 2)
+                     && (Ability.getSk Ability.SkShine actorMaxSk > 2
+                         || condNonStealthyThreatAdj || null threatAdj)
             return $!
-              -- This is a part of the condition for @flee@ in @PickActionM@.
               not condFastThreatAdj
-              && if | condThreat 1 ->
+              && fleeingMakesSense
+              && if | condAnyHarmfulFoeAdj ->
                       not condCanMelee
                       || condManyThreatAdj && not condSupport1 && not condSolo
-                    | not condInMelee
-                      && (condThreat 2 || condThreat 5 && canFleeFromLight) ->
-                      not condCanMelee
-                      || not condSupport3 && not condSolo
-                         && not heavilyDistressed
-                    -- Not used: | condThreat 5 ...
-                    -- because actor should be picked anyway, to try to melee.
-                    | otherwise ->
-                      not condInMelee
-                      && heavilyDistressed
-                      -- Different from @PickActionM@:
-                      && not (EM.member aid fleeD)
-                        -- Make him a leader even if can't delight, etc.
-                        -- because he may instead take off light or otherwise
-                        -- cope with being pummeled by projectiles.
-                        -- He is still vulnerable, just not necessarily needs
-                        -- to flee, but may cover himself otherwise.
-                        -- && (not condCanProject || canFleeFromLight)
+                    | condInMelee -> False
+                    | heavilyDistressed -> True
+                        -- Different from @PickActionM@:
+                        -- If under fire, do something quickly, always,
+                        -- because the actor clearly vulnerable,
+                        -- but don't make a leader only because threats close.
+                    | otherwise -> False
               && condCanFlee
-          actorFled ((aid, _), _) = EM.member aid fleeD
+          actorFled ((aid, _), _) = recentlyFled aid
           actorHearning (_, TgtAndPath{ tapTgt=TPoint TEnemyPos{} _ _
                                       , tapPath=Nothing }) =
             return False
@@ -166,29 +179,50 @@ pickActorToMove maidToAvoid = do
             | pathLen <= 2 =
             return False  -- noise probably due to fleeing target
           actorHearning ((_aid, b), _) = do
-            allFoes <- getsState $ foeRegularList side (blid b)
-            let closeFoes = filter ((<= 3) . chessDist (bpos b) . bpos) allFoes
+            let closeFoes = filter ((<= 3) . chessDist (bpos b) . bpos . snd)
+                                   foeAssocs
                 actorHears = deltasHears (bcalmDelta b)
             return $! actorHears  -- e.g., actor hears an enemy
                       && null closeFoes  -- the enemy not visible; a trap!
-          -- AI has to be prudent and not lightly waste leader for meleeing,
-          -- even if his target is distant
-          actorMeleeing ((aid, _), _) = condAnyFoeAdjM aid
+          -- AI has to be prudent and not lightly waste leader for meleeing.
+          actorMeleeing ((aid, _), _) =
+            getsState $ anyHarmfulFoeAdj actorMaxSkills aid
       (oursVulnerable, oursSafe) <- partitionM actorVulnerable oursTgt
       let (oursFled, oursNotFled) = partition actorFled oursSafe
-      (oursMeleeing, oursNotMeleeing) <- partitionM actorMeleeing oursNotFled
+      (oursMeleeingRaw, oursNotMeleeingRaw) <-
+         partitionM actorMeleeing oursNotFled
+      let actorMeleeingCanDisplace ( (aid, sb)
+                                   , TgtAndPath{tapTgt=TEnemy target} ) = do
+            tb <- getsState $ getActorBody target
+            let actorMaxSk = actorMaxSkills EM.! target
+            dEnemy <- getsState $ dispEnemy aid target actorMaxSk
+            -- Some usual conditions ignored, because transient or rare.
+            return $! checkAdjacent sb tb && dEnemy
+          actorMeleeingCanDisplace _ = return False
+      (oursMeleeingCanDisplace, oursMeleeing) <-
+         partitionM actorMeleeingCanDisplace oursMeleeingRaw
+      let adjStash
+            ( (_, b)
+            , TgtAndPath{tapTgt=TPoint TStash{} lid pos} ) =
+                lid == arena
+                && adjacent pos (bpos b)
+                && isNothing (posToBigLvl pos lvl)
+          adjStash _ = False
+          (oursAdjStash, oursNotMeleeing) =
+            partition adjStash oursNotMeleeingRaw
       (oursHearing, oursNotHearing) <- partitionM actorHearning oursNotMeleeing
       let actorRanged ((aid, body), _) =
             not $ actorCanMelee actorMaxSkills aid body
           targetTEnemy (_, TgtAndPath{tapTgt=TEnemy _}) = True
-          targetTEnemy
-            ( (_, b)
-            , TgtAndPath{tapTgt=TPoint (TEnemyPos _) lid _} ) =
-              lid == blid b
+          targetTEnemy (_, TgtAndPath{tapTgt=TPoint TEnemyPos{} lid _}) =
+            lid == arena
+          targetTEnemy ((_, b), TgtAndPath{tapTgt=TPoint TStash{} lid pos}) =
+            lid == arena && pos /= bpos b
+              -- stashes as crucial as enemies. except when guarding them
           targetTEnemy _ = False
           actorNoSupport ((aid, _), _) = do
-            threatDistL <- getsState $ meleeThreatDistList aid
-            condSupport2 <- condSupport 2 aid
+            threatDistL <- getsState $ meleeThreatDistList foeAssocs aid
+            condSupport2 <- condSupport friendAssocs 2 aid
             let condThreat n = not $ null $ takeWhile ((<= n) . fst) threatDistL
             -- If foes far, friends may still come, so we let him move.
             -- The net effect is that lone heroes close to foes freeze
@@ -196,6 +230,9 @@ pickActorToMove maidToAvoid = do
             return $! condThreat 5 && not condSupport2
           (oursRanged, oursNotRanged) = partition actorRanged oursNotHearing
           (oursTEnemyAll, oursOther) = partition targetTEnemy oursNotRanged
+          notSwapReady ((_, b), TgtAndPath{tapTgt=TPoint TStash{} lid pos}) _ =
+            lid == arena && pos == bpos b
+              -- not ready to follow goal if already guarding the stash
           notSwapReady abt@((_, b), _)
                        (ab2, Just t2@TgtAndPath{tapPath=
                                        Just AndPath{pathList=q : _}}) =
@@ -230,23 +267,30 @@ pickActorToMove maidToAvoid = do
             else (oursNoSupportRaw, oursSupportRaw)
           (oursBlocked, oursPos) =
             partition targetBlocked $ oursRanged ++ oursOther
+          guarding ((_, b), Just TgtAndPath{tapTgt=TPoint TStash{} lid pos}) =
+            lid == arena && pos == bpos b
+          guarding _ = False
+          -- Don't try to include a stash guard in formation, even if attacking
+          -- or being attacked. Attackers would be targetted anyway.
+          oursNotSleepingNorGuarding = filter (not . guarding) oursTgtRaw
           -- Lower overhead is better.
           overheadOurs :: ((ActorId, Actor), TgtAndPath) -> Int
-          overheadOurs ((aid, _), TgtAndPath{tapPath=Nothing}) =
-            100 + if aid == oldAid then 1 else 0
-          overheadOurs
-            abt@( (aid, b)
-                , TgtAndPath{tapPath=Just AndPath{pathLen=d,pathGoal}} ) =
+          overheadOurs (_, TgtAndPath{tapPath=Nothing}) = 100
+          overheadOurs ((_, b), TgtAndPath{tapTgt=TPoint TStash{} lid pos})
+            | lid == arena && pos == bpos b = 200  -- guarding, poor choice
+          overheadOurs abt@( (aid, b)
+                           , TgtAndPath{tapPath=Just AndPath{pathLen=d, ..}} ) =
             -- Keep proper formation. Too dense and exploration takes
             -- too long; too sparse and actors fight alone.
             -- Note that right now, while we set targets separately for each
             -- hero, perhaps on opposite borders of the map,
             -- we can't help that sometimes heroes are separated.
-            let maxSpread = 3 + length oursNotSleeping
+            let maxSpread = 3 + length oursNotSleepingNorGuarding
                 lDist p = [ chessDist (bpos b2) p
-                          | (aid2, b2) <- oursNotSleeping, aid2 /= aid]
+                          | ((aid2, b2), _) <- oursNotSleepingNorGuarding
+                          , aid2 /= aid ]
                 pDist p = let ld = lDist p
-                          in assert (not $ null ld) $ minimum ld
+                          in if null ld then 0 else minimum ld
                 aidDist = pDist (bpos b)
                 -- Negative, if the goal gets us closer to the party.
                 diffDist = pDist pathGoal - aidDist
@@ -255,27 +299,51 @@ pickActorToMove maidToAvoid = do
                 formationValue =
                   sign * (abs diffDist `max` maxSpread)
                   * (aidDist `max` maxSpread) ^ (2 :: Int)
-                fightValue | targetTEnemy abt =
-                  - fromEnum (bhp b `div` (10 * oneM))
-                           | otherwise = 0
-            in formationValue `div` 3 + fightValue
-               + (if targetBlocked abt then 5 else 0)
+                targetsEnemy = targetTEnemy abt
+                fightValue = if targetsEnemy
+                             then - fromEnum (bhp b `div` (10 * oneM))
+                             else 0
+                isLit pos = Tile.isLit coTileSpeedup (lvl `at` pos)
+                  -- solid tiles ignored, because not obvious if dark
+                  -- after removed
+                actorMaxSk = actorMaxSkills EM.! aid
+                actorShines = Ability.getSk Ability.SkShine actorMaxSk > 0
+                stepsIntoLight =
+                  not actorShines
+                  && not (isLit $ bpos b)
+                  && case pathList of
+                    [] -> False
+                    q : _ -> isLit q
+                      -- shortest path is through light even though may
+                      -- sidestep through dark in @chase@ or @flee@
+            in formationValue `div` 3
+               + fightValue
                + (case d of
-                    0 -> -400 -- do your thing ASAP and retarget
-                    1 -> -200 -- prevent others from occupying the tile
+                    0 -> -400  -- do your thing ASAP and retarget
+                    1 | not targetsEnemy -> -200
+                      -- prevent others from trying to occupy the tile;
+                      -- TStash that obscures a foe correctly handled here
                     _ -> if d < 8 then d `div` 4 else 2 + d `div` 10)
-               + (if aid == oldAid then 1 else 0)
-          positiveOverhead sk =
-            let ov = 200 - overheadOurs sk
-            in if ov <= 0 then 1 else ov
-          candidates = [ oursVulnerable
+               + (if aid == oldAid then 0 else 10)
+               + (if stepsIntoLight then 30 else 0)
+          -- Overheads above @maxBoundInt32@ are unlikely (and unsuppored in JS)
+          -- and also capping the value does not distort the choice too much.
+          positiveOverhead abt =
+            min maxBoundInt32 $ max 1 $ 200 - overheadOurs abt
+          candidates = [ oursAdjStash
+                       , oursVulnerable
                        , oursSupport
                        , oursNoSupport
                        , oursPos
-                       , oursFled  -- if just fled, keep him safe, out of action
-                       , oursMeleeing ++ oursTEnemyBlocked
-                           -- make melee a leader to displace or at least melee
-                           -- without overhead if all others blocked
+                       , oursFled  -- if just fled, but not vulnerable,
+                                   -- keep him passive and safe, out of action
+                       , oursMeleeingCanDisplace
+                           -- prefer melee actors displacing than blocked
+                           -- actors trying to walk around them
+                       , oursTEnemyBlocked
+                           -- prefer blocked actors trying to walk around
+                           -- even if that causes overhead for the meleeing
+                       , oursMeleeing
                        , oursHearing
                        , oursBlocked
                        ]
@@ -288,34 +356,32 @@ pickActorToMove maidToAvoid = do
           modifyClient $ updateLeader aid s
           -- When you become a leader, stop following old leader, but follow
           -- his target, if still valid, to avoid distraction.
-          condInMelee <- condInMeleeM $ blid b
-          when (ftactic (gplayer fact)
-                `elem` [Ability.TFollow, Ability.TFollowNoItems]
+          when (gdoctrine fact `elem` [Ability.TFollow, Ability.TFollowNoItems]
                 && not condInMelee) $
-            void $ refreshTarget (aid, b)
+            void $ refreshTarget foeAssocs friendAssocs (aid, b)
           return aid
         _ -> return oldAid
 
--- | Inspect the tactics of the actor and set his target according to it.
-setTargetFromTactics :: MonadClient m => ActorId -> m ()
-{-# INLINE setTargetFromTactics #-}
-setTargetFromTactics oldAid = do
+-- | Inspect the doctrines of the actor and set his target according to it.
+setTargetFromDoctrines :: MonadClient m
+                        => [(ActorId, Actor)] -> [(ActorId, Actor)] -> ActorId
+                        -> m ()
+setTargetFromDoctrines foeAssocs friendAssocs oldAid = do
   mleader <- getsClient sleader
   let !_A = assert (mleader /= Just oldAid) ()
   oldBody <- getsState $ getActorBody oldAid
   moldTgt <- getsClient $ EM.lookup oldAid . stargetD
-  condInMelee <- condInMeleeM $ blid oldBody
   let side = bfid oldBody
       arena = blid oldBody
   fact <- getsState $ (EM.! side) . sfactionD
-  let explore = void $ refreshTarget (oldAid, oldBody)
+  let explore = void $ refreshTarget foeAssocs friendAssocs (oldAid, oldBody)
       setPath mtgt = case (mtgt, moldTgt) of
         (Nothing, _) -> return False
         ( Just TgtAndPath{tapTgt=leaderTapTgt},
           Just TgtAndPath{tapTgt=oldTapTgt,tapPath=Just oldTapPath} )
           | leaderTapTgt == oldTapTgt  -- targets agree
             && bpos oldBody == pathSource oldTapPath -> do  -- nominal path
-            void $ refreshTarget (oldAid, oldBody)
+            void $ refreshTarget foeAssocs friendAssocs (oldAid, oldBody)
             return True  -- already on target
         (Just TgtAndPath{tapTgt=leaderTapTgt}, _) -> do
             tap <- createPath oldAid leaderTapTgt
@@ -326,7 +392,7 @@ setTargetFromTactics oldAid = do
                   cli {stargetD = EM.insert oldAid tap (stargetD cli)}
                 return True
       follow = case mleader of
-        -- If no leader at all (forced @TFollow@ tactic on an actor
+        -- If no leader at all (forced @TFollow@ doctrine on an actor
         -- from a leaderless faction), fall back to @TExplore@.
         Nothing -> explore
         _ | bwatch oldBody == WSleep ->
@@ -334,7 +400,9 @@ setTargetFromTactics oldAid = do
           explore
         Just leader -> do
           onLevel <- getsState $ memActor leader arena
-          -- If leader not on this level, fall back to @TExplore@.
+          condInMelee <- condInMeleeM arena
+          -- If leader not on this level or if we are meleeing,
+          -- and so following is not important, fall back to @TExplore@.
           if not onLevel || condInMelee then explore
           else do
             -- Copy over the leader's target, if any, or follow his position.
@@ -347,7 +415,7 @@ setTargetFromTactics oldAid = do
               unless nonEnemyPathSet
                 -- If no path even to the leader himself, explore.
                 explore
-  case ftactic $ gplayer fact of
+  case gdoctrine fact of
     Ability.TExplore -> explore
     Ability.TFollow -> follow
     Ability.TFollowNoItems -> follow
