@@ -9,6 +9,7 @@ module Game.LambdaHack.Client.BfsM
   , furthestKnown, closestUnknown, closestSmell
   , FleeViaStairsOrEscape(..)
   , embedBenefit, closestTriggers, condEnoughGearM, closestItems, closestFoes
+  , closestStashes, oursExploringAssocs, closestHideout
 #ifdef EXPOSE_INTERNAL
   , unexploredDepth, updatePathFromBfs
 #endif
@@ -20,7 +21,6 @@ import Game.LambdaHack.Core.Prelude
 
 import qualified Data.EnumMap.Strict as EM
 import qualified Data.EnumSet as ES
-import           Data.Ord
 import           Data.Word
 
 import           Game.LambdaHack.Client.Bfs
@@ -35,24 +35,27 @@ import qualified Game.LambdaHack.Common.ItemAspect as IA
 import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Level
 import           Game.LambdaHack.Common.MonadStateRead
+import           Game.LambdaHack.Common.Perception
+import           Game.LambdaHack.Common.Point
+import qualified Game.LambdaHack.Common.PointArray as PointArray
 import           Game.LambdaHack.Common.State
 import qualified Game.LambdaHack.Common.Tile as Tile
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Types
 import           Game.LambdaHack.Common.Vector
+import qualified Game.LambdaHack.Content.CaveKind as CK
+import           Game.LambdaHack.Content.FactionKind
 import qualified Game.LambdaHack.Content.ItemKind as IK
-import           Game.LambdaHack.Content.ModeKind
 import           Game.LambdaHack.Content.RuleKind
 import           Game.LambdaHack.Content.TileKind (isUknownSpace)
-import           Game.LambdaHack.Common.Point
-import qualified Game.LambdaHack.Common.PointArray as PointArray
 import           Game.LambdaHack.Core.Random
 import qualified Game.LambdaHack.Definition.Ability as Ability
 import           Game.LambdaHack.Definition.Defs
 
 invalidateBfsAid :: MonadClient m => ActorId -> m ()
 invalidateBfsAid aid =
-  modifyClient $ \cli -> cli {sbfsD = EM.insert aid BfsInvalid (sbfsD cli)}
+  modifyClient $ \cli ->
+    cli {sbfsD = EM.adjust (const BfsInvalid) aid (sbfsD cli)}
 
 invalidateBfsPathAid :: MonadClient m => ActorId -> m ()
 invalidateBfsPathAid aid = do
@@ -60,21 +63,27 @@ invalidateBfsPathAid aid = do
       f (BfsAndPath bfsArr _) = BfsAndPath bfsArr EM.empty
   modifyClient $ \cli -> cli {sbfsD = EM.adjust f aid (sbfsD cli)}
 
+-- Even very distant actors affected, e.g., when a hidden door found in a wall.
 invalidateBfsLid :: MonadClient m => LevelId -> m ()
 invalidateBfsLid lid = do
-  side <- getsClient sside
-  let f (_, b) = blid b == lid && bfid b == side && not (bproj b)
-  as <- getsState $ filter f . EM.assocs . sactorD
-  mapM_ (invalidateBfsAid . fst) as
+  lvl <- getLevel lid
+  -- No need to filter, because foes won't be in our BFS map and looking up
+  -- in our BFS map is faster than in all actors map.
+  mapM_ invalidateBfsAid $ EM.elems $ lbig lvl
 
-invalidateBfsPathLid :: MonadClient m => LevelId -> Point -> m ()
-invalidateBfsPathLid lid pos = do
-  side <- getsClient sside
-  let f (_, b) = blid b == lid && bfid b == side && not (bproj b)
-                 && chessDist pos (bpos b) < actorsAvoidedDist
-                      -- rough approximation, but kicks in well before blockage
-  as <- getsState $ filter f . EM.assocs . sactorD
-  mapM_ (invalidateBfsPathAid . fst) as
+-- We invalidate, but not when actors move, since they are likely to move
+-- out of the way in time. We only do, when they appear or disappear,
+-- because they may be immobile or too close to move away before we get there.
+-- We also don't consider far actors, since they are likely to disappear
+-- again or to be far from our path. If they close enough to be lit
+-- by our light, or one step further, that's worth taking seriously.
+invalidateBfsPathLid :: MonadClient m => Actor -> m ()
+invalidateBfsPathLid body = do
+  lvl <- getLevel $ blid body
+  let close (p, _) = chessDist p (bpos body) <= 3  -- heuristic
+  -- No need to filter more, because foes won't be in our BFS map and looking up
+  -- in our BFS map is faster than in all actors map.
+  mapM_ (invalidateBfsPathAid . snd) $ filter close $ EM.assocs $ lbig lvl
 
 invalidateBfsAll :: MonadClient m => m ()
 invalidateBfsAll =
@@ -113,18 +122,24 @@ updatePathFromBfs canMove bfsAndPathOld aid !target = do
   if not canMove
   then return (bfsArr, Nothing)
   else do
-    b <- getsState $ getActorBody aid
-    let lid = blid b
+    getActorB <- getsState $ flip getActorBody
+    let b = getActorB aid
+    fact <- getsState $ (EM.! bfid b) . sfactionD
     seps <- getsClient seps
     salter <- getsClient salter
-    lvl <- getLevel lid
-    let !lalter = salter EM.! lid
+    lvl <- getLevel (blid b)
+    let !lalter = salter EM.! blid b
         fovLit p = Tile.isLit coTileSpeedup $ PointArray.fromUnboxRep
                                             $ ltile lvl `PointArray.accessI` p
+        addFoeVicinity (p, aid2) =
+          let b2 = getActorB aid2
+          in if isFoe (bfid b) fact (bfid b2)
+             then p : vicinityUnsafe p
+             else [p]
+        bigAdj = ES.fromList $ concatMap addFoeVicinity $ EM.assocs
+                 $ EM.delete source $ lbig lvl  -- don't sidestep oneself
         !source = bpos b
-        !mpath =
-          findPathBfs (EM.delete source $ lbig lvl)  -- don't sidestep oneself
-                      lalter fovLit source target seps bfsArr
+        !mpath = findPathBfs bigAdj lalter fovLit source target seps bfsArr
         !bfsPath =
           maybe oldBfsPath (\path -> EM.insert target path oldBfsPath) mpath
         bap = BfsAndPath bfsArr bfsPath
@@ -169,9 +184,9 @@ getCachePath :: MonadClient m => ActorId -> Point -> m (Maybe AndPath)
 getCachePath aid target = do
   b <- getsState $ getActorBody aid
   let source = bpos b
-  if | source == target ->
-       return $ Just $ AndPath (bpos b) [] target 0  -- speedup
-     | otherwise -> snd <$> getCacheBfsAndPath aid target
+  if source == target
+  then return $ Just $ AndPath (bpos b) [] target 0  -- speedup
+  else snd <$> getCacheBfsAndPath aid target
 
 createPath :: MonadClient m => ActorId -> Target -> m TgtAndPath
 createPath aid tapTgt = do
@@ -180,22 +195,25 @@ createPath aid tapTgt = do
   lvl <- getLevel $ blid b
   let stopAtUnwalkable tapPath@(Just AndPath{..}) =
         let (walkable, rest) =
-              -- Unknown tiles are not walkable, so path stops at first such.
+              -- Unknown tiles are not walkable, so path stops just before.
               -- which is good, because by the time actor reaches the tile,
-              -- it is known and target is recalculated with new info.
+              -- it is known and target is recalculated with new info,
+              -- perhaps sidestepping the tile, e.g., if explosive.
               span (Tile.isWalkable coTileSpeedup . at lvl) pathList
         in case rest of
+          _ | null walkable -> TgtAndPath{..}
           [] -> TgtAndPath{..}
           [g] | g == pathGoal -> TgtAndPath{..}
+            -- the exception is when the tile is explicitly targeted
           newGoal : _ ->
             let newTgt = TPoint TBlock (blid b) newGoal
                 newPath = AndPath{ pathSource = bpos b
-                                 , pathList = walkable ++ [newGoal]
+                                 , pathList = walkable  -- no @newGoal@
                                  , pathGoal = newGoal
                                  , pathLen = length walkable + 1 }
             in TgtAndPath{tapTgt = newTgt, tapPath = Just newPath}
       stopAtUnwalkable Nothing = TgtAndPath{tapTgt, tapPath=Nothing}
-  mpos <- getsState $ aidTgtToPos aid (blid b) (Just tapTgt)
+  mpos <- getsState $ aidTgtToPos (Just aid) (blid b) (Just tapTgt)
   case mpos of
     Nothing -> return TgtAndPath{tapTgt, tapPath=Nothing}
     Just p -> do
@@ -219,11 +237,10 @@ condBFS aid = do
                 || Ability.getSk Ability.SkProject actorMaxSk > 0
   smarkSuspect <- getsClient smarkSuspect
   fact <- getsState $ (EM.! side) . sfactionD
-  let underAI = isAIFact fact
-      -- Under UI, playing a hero party, we let AI set our target each
-      -- turn for henchmen that can't move and can't alter, usually to TUnknown.
-      -- This is rather useless, but correct.
-      enterSuspect = smarkSuspect > 0 || underAI
+  let -- Under UI, playing a hero party, we let AI set our target each
+      -- turn for non-pointmen that can't move and can't alter,
+      -- usually to TUnknown. This is rather useless, but correct.
+      enterSuspect = smarkSuspect > 0 || gunderAI fact
       skill | enterSuspect = alterSkill  -- dig and search as skill allows
             | otherwise = 0  -- only walkable tiles
   return (canMove, skill)  -- keep it lazy
@@ -249,13 +266,13 @@ furthestKnown aid = do
 -- all dungeon styles would be complex and would slow down the engine.
 --
 -- If the level has inaccessible open areas (at least from the stairs AI used)
--- the level will be nevertheless here finally marked explored,
+-- the level will be nevertheless here finally labeled as explored,
 -- to enable transition to other levels.
 -- We should generally avoid such levels, because digging and/or trying
 -- to find other stairs leading to disconnected areas is not KISS
 -- so we don't do this in AI, so AI is at a disadvantage.
 --
--- If the closest unknown is more than 126 tiles away from the targetting
+-- If the closest unknown is more than 126 tiles away from the targeting
 -- actor, the level will marked as explored. We could complicate the code
 -- and not mark if the unknown is too far as opposed to inaccessible,
 -- but then if it is both too distant and inaccessible, AI would be
@@ -308,7 +325,7 @@ closestSmell aid = do
     _ -> do
       bfs <- getCacheBfs aid
       let ts = mapMaybe (\x@(p, _) -> fmap (,x) (accessBfs bfs p)) smells
-      return $! sortBy (comparing (fst &&& absoluteTimeNegate . snd . snd)) ts
+      return $! sortOn (fst &&& absoluteTimeNegate . snd . snd) ts
 
 data FleeViaStairsOrEscape =
     ViaStairs
@@ -325,17 +342,26 @@ embedBenefit :: MonadClientRead m
              -> [(Point, ItemBag)]
              -> m [(Double, (Point, ItemBag))]
 embedBenefit fleeVia aid pbags = do
-  COps{coTileSpeedup} <- getsState scops
+  COps{cocave, coTileSpeedup} <- getsState scops
   dungeon <- getsState sdungeon
   explored <- getsClient sexplored
   b <- getsState $ getActorBody aid
+  fact <- getsState $ (EM.! bfid b) . sfactionD
+  lvl <- getLevel (blid b)
+  oursExploring <- getsState $ oursExploringAssocs (bfid b)
+  let oursExploringLid =
+        filter (\(_, body) -> blid body == blid b) oursExploring
+      spawnFreqs = CK.cactorFreq $ okind cocave $ lkind lvl
+      hasGroup grp = fromMaybe 0 (lookup grp spawnFreqs) > 0
+      lvlSpawnsUs = any (hasGroup . fst) $ filter ((> 0) . snd)
+                                         $ fgroups (gkind fact)
   actorSk <- if fleeVia `elem` [ViaAnything, ViaExit]
                   -- targeting, possibly when not a leader
              then getsState $ getActorMaxSkills aid
              else currentSkillsClient aid
   let alterSkill = Ability.getSk Ability.SkAlter actorSk
-  fact <- getsState $ (EM.! bfid b) . sfactionD
-  lvl <- getLevel (blid b)
+  condOurAdj <- getsState $ any (\(_, b2) -> isFriend (bfid b) fact (bfid b2))
+                            . adjacentBigAssocs b
   unexploredTrue <- unexploredDepth True (blid b)
   unexploredFalse <- unexploredDepth False (blid b)
   condEnoughGear <- condEnoughGearM aid
@@ -356,8 +382,8 @@ embedBenefit fleeVia aid pbags = do
         Just IK.Escape{} ->
           -- Escape (or guard) only after exploring, for high score, etc.
           let escapeOrGuard =
-                fcanEscape (gplayer fact)
-                || fleeVia `elem` [ViaExit]  -- target to guard after explored
+                fcanEscape (gkind fact)
+                || fleeVia == ViaExit  -- target to guard after explored
           in if fleeVia `elem` [ViaAnything, ViaEscape, ViaExit]
                 && escapeOrGuard
                 && allExplored
@@ -371,11 +397,23 @@ embedBenefit fleeVia aid pbags = do
               aiCond = if unexpForth
                        then easier && condEnoughGear
                             || (not unexpBack || easier) && lidExplored
-                       else easier && allExplored && null (lescape lvl)
+                       else not unexpBack && easier && allExplored
+                            && null (lescape lvl)
               -- Prefer one direction of stairs, to team up
               -- and prefer embed (may, e.g., create loot) over stairs.
               v = if aiCond then if easier then 10 else 1 else 0
+              guardingStash = case gstash fact of
+                Nothing -> False
+                Just (lid, p) ->
+                  lid == blid b
+                  && (length oursExploring > 1
+                      || lvlSpawnsUs)
+                  && (length oursExploringLid <= 1
+                        -- not @==@ in case guard temporarily nonmoving
+                      || p == bpos b && not condOurAdj)
+                           -- don't leave the post; let the others explore
           in case fleeVia of
+            _ | guardingStash -> 0
             ViaStairsUp | up -> 1
             ViaStairsDown | not up -> 1
             ViaStairs -> v
@@ -394,9 +432,12 @@ embedBenefit fleeVia aid pbags = do
           else 0
       underFeet p = p == bpos b  -- if enter and alter, be more permissive
       -- Only actors with high enough @SkAlter@ can trigger terrain.
-      -- If apply skill not high enough for embedded items, AI will only
-      -- guard such tiles, assuming they must be advanced and so crucial.
-      f (p, _) = underFeet p || alterSkill >= fromEnum (alterMinSkill p)
+      -- Blocking actors and items not checked, because they can be moved
+      -- before the actor gets to the location, or after.
+      f (p, _) = underFeet p
+                 || alterSkill >= fromEnum (alterMinSkill p)
+                 || Tile.isSuspect coTileSpeedup (lvl `at` p)
+                    && alterSkill >= 2
       benFeats = map (\pbag -> (bens pbag, pbag)) $ filter f pbags
       considered (benefitAndSacrifice, (p, _bag)) =
         benefitAndSacrifice > 0
@@ -412,7 +453,7 @@ embedBenefit fleeVia aid pbags = do
 closestTriggers :: MonadClient m => FleeViaStairsOrEscape -> ActorId
                 -> m [(Int, (Point, (Point, ItemBag)))]
 closestTriggers fleeVia aid = do
-  COps{corule=RuleContent{rXmax, rYmax}} <- getsState scops
+  COps{corule=RuleContent{rWidthMax, rHeightMax}} <- getsState scops
   b <- getsState $ getActorBody aid
   lvl <- getLevel (blid b)
   let pbags = EM.assocs $ lembed lvl
@@ -425,12 +466,13 @@ closestTriggers fleeVia aid = do
   -- OTOH, siege of stairs or escapes is more effective.
   bfs <- getCacheBfs aid
   let vicTrigger (cid, (p0, bag)) =
-        map (\p -> (cid, (p, (p0, bag)))) $ vicinityBounded rXmax rYmax p0
+        map (\p -> (cid, (p, (p0, bag))))
+            (vicinityBounded rWidthMax rHeightMax p0)
       vicAll = concatMap vicTrigger efeat
   return $!
     let mix (benefit, ppbag) dist =
           let maxd = subtractBfsDistance maxBfsDistance apartBfs
-              v = fromIntegral $ (1 + maxd - dist) ^ (2 :: Int)
+              v = intToDouble $ maxd `div` (dist + 1)
           in (ceiling $ benefit * v, ppbag)
     in mapMaybe (\bpp@(_, (p, _)) ->
          mix bpp <$> accessBfs bfs p) vicAll
@@ -438,22 +480,21 @@ closestTriggers fleeVia aid = do
 -- | Check whether the actor has enough gear to go look for enemies.
 -- We assume weapons in equipment are better than any among organs
 -- or at least provide some essential diversity.
--- Disabled if, due to tactic, actors follow leader and so would
--- repeatedly move towards and away form stairs at leader change,
+-- Disabled if, due to doctrine, actors follow leader and so would
+-- repeatedly move towards and away from stairs at leader change,
 -- depending on current leader's gear.
 -- Number of items of a single kind is ignored, because variety is needed.
 condEnoughGearM :: MonadClientRead m => ActorId -> m Bool
 condEnoughGearM aid = do
   b <- getsState $ getActorBody aid
   fact <- getsState $ (EM.! bfid b) . sfactionD
-  let followTactic = ftactic (gplayer fact)
-                     `elem` [Ability.TFollow, Ability.TFollowNoItems]
+  let followDoctrine =
+        gdoctrine fact `elem` [Ability.TFollow, Ability.TFollowNoItems]
   eqpAssocs <- getsState $ fullAssocs aid [CEqp]
-  invAssocs <- getsState $ getActorAssocs aid CInv
-  return $ not followTactic  -- keep it lazy
+  return $ not followDoctrine  -- keep it lazy
            && (any (IA.checkFlag Ability.Meleeable
                     . aspectRecordFull . snd) eqpAssocs
-               || length eqpAssocs + length invAssocs >= 5)
+               || length eqpAssocs >= 3)
 
 unexploredDepth :: MonadClientRead m => Bool -> LevelId -> m Bool
 unexploredDepth !up !lidCurrent = do
@@ -471,22 +512,38 @@ unexploredDepth !up !lidCurrent = do
 -- | Closest (wrt paths) items.
 closestItems :: MonadClient m => ActorId -> m [(Int, (Point, ItemBag))]
 closestItems aid = do
-  actorMaxSk <- getsState $ getActorMaxSkills aid
-  if Ability.getSk Ability.SkMoveItem actorMaxSk <= 0 then return []
-  else do
-    body <- getsState $ getActorBody aid
-    Level{lfloor} <- getLevel $ blid body
-    if EM.null lfloor then return [] else do
-      bfs <- getCacheBfs aid
-      let mix pbag dist =
-            let maxd = subtractBfsDistance maxBfsDistance apartBfs
-                -- Beware of overflowing 32-bit integers.
-                -- Here distance is the only factor influencing frequency.
-                -- Whether item is desirable is checked later on.
-                v = (maxd * 10) `div` (dist + 1)
-            in (v, pbag)
-      return $! mapMaybe (\(p, bag) ->
-        mix (p, bag) <$> accessBfs bfs p) (EM.assocs lfloor)
+  body <- getsState $ getActorBody aid
+  Level{lfloor, lbig} <- getLevel $ blid body
+  factionD <- getsState sfactionD
+  per <- getPerFid $ blid body
+  let canSee p = ES.member p (totalVisible per)
+  -- Don't consider items at any stash location that an actor stands over
+  -- or can stand over, but it's out of our LOS.
+  -- In case of the own stash, don't consider regardless of actors and LOS.
+  -- Own stash items are already owned, enemy stash is already targetted
+  -- and allied or neutral stashes with actors on top are unlikely
+  -- to be vacated and cause AI to wonder around forever or look up,
+  -- leave, return hopeful, find a guard, repeat.
+  let stashes = map (second gstash) $ EM.assocs factionD
+      stashToRemove :: (FactionId, Maybe (LevelId, Point)) -> [Point]
+      stashToRemove (fid, Just (lid, pos))
+        | lid == blid body
+          && (fid == bfid body || pos `EM.member` lbig || not (canSee pos)) =
+            [pos]
+      stashToRemove _ = []
+      stashesToRemove = ES.fromList $ concatMap stashToRemove stashes
+      lfloorBarStashes = EM.withoutKeys lfloor stashesToRemove
+  if EM.null lfloorBarStashes then return [] else do
+    bfs <- getCacheBfs aid
+    let mix pbag dist =
+          let maxd = subtractBfsDistance maxBfsDistance apartBfs
+              -- Beware of overflowing 32-bit integers.
+              -- Here distance is the only factor influencing frequency.
+              -- Whether item is desirable is checked later on.
+              v = (maxd * 10) `div` (dist + 1)
+          in (v, pbag)
+    return $! mapMaybe (\(p, bag) ->
+      mix (p, bag) <$> accessBfs bfs p) (EM.assocs lfloorBarStashes)
 
 -- | Closest (wrt paths) enemy actors.
 closestFoes :: MonadClient m
@@ -498,3 +555,74 @@ closestFoes foes aid =
       bfs <- getCacheBfs aid
       let ds = mapMaybe (\x@(_, b) -> fmap (,x) (accessBfs bfs (bpos b))) foes
       return $! sortBy (comparing fst) ds
+
+-- | Closest (wrt paths) enemy or our unguarded stash locations. If it's ours,
+-- we want to guard it, it enemy, loot it. Neutral and friendly stashes
+-- not chased to avoid loops of bloodless takeovers.
+closestStashes :: MonadClient m => ActorId -> m [(Int, (FactionId, Point))]
+closestStashes aid = do
+  COps{cocave} <- getsState scops
+  factionD <- getsState sfactionD
+  b <- getsState $ getActorBody aid
+  lvl <- getLevel (blid b)
+  oursExploring <- getsState $ oursExploringAssocs (bfid b)
+  let fact = factionD EM.! bfid b
+      spawnFreqs = CK.cactorFreq $ okind cocave $ lkind lvl
+      hasGroup grp = fromMaybe 0 (lookup grp spawnFreqs) > 0
+      lvlSpawnsUs = any (hasGroup . fst) $ filter ((> 0) . snd)
+                                         $ fgroups (gkind fact)
+      qualifyStash (fid2, Faction{gstash}) = case gstash of
+        Nothing -> Nothing
+        Just (lid, pos) ->
+          -- The condition below is more strict that in @updateTgt@
+          -- to avoid loops by changing target of actor displacing
+          -- and walking over stash to @TStash@.
+          if lid == blid b
+             && (fid2 == bfid b
+                 && isNothing (posToBigLvl pos lvl)  -- unguarded
+                 && (length oursExploring > 1  -- other actors able to explore
+                     || lvlSpawnsUs)  -- or future spawned will be able
+                 || isFoe (bfid b) fact fid2)
+          then Just (fid2, pos)
+          else Nothing
+  case mapMaybe qualifyStash $ EM.assocs factionD of
+    [] -> return []
+    stashes -> do
+      bfs <- getCacheBfs aid
+      let ds = mapMaybe (\x@(_, pos) -> fmap (,x) (accessBfs bfs pos)) stashes
+      return $! sortBy (comparing fst) ds
+
+oursExploringAssocs :: FactionId -> State -> [(ActorId, Actor)]
+oursExploringAssocs fid s =
+  let f (!aid, !b) = bfid b == fid
+                     && not (bproj b)
+                     && bhp b > 0  -- dead can stay forever on a frozen level
+                     && (bwatch b `elem` [WSleep, WWake]
+                           -- if asleep, probably has walking skill normally;
+                           -- when left alone will wake up and guard or explore
+                        || let actorMaxSk = sactorMaxSkills s EM.! aid
+                           in Ability.getSk Ability.SkMove actorMaxSk > 0
+                              || Ability.getSk Ability.SkMove actorMaxSk < -50)
+                                   -- a hacky way to rule out tmp immobile
+  in filter f $ EM.assocs $ sactorD s
+
+-- | Find the nearest walkable position in dark, if any. Deterministic,
+-- to let all friends gather up and defend in the same shelter.
+-- Ignore position underfoot.
+closestHideout :: MonadClient m => ActorId -> m (Maybe (Point, Int))
+closestHideout aid = do
+  COps{coTileSpeedup} <- getsState scops
+  b <- getsState $ getActorBody aid
+  lvl <- getLevel (blid b)
+  bfs <- getCacheBfs aid
+  let minHideout :: (Point, BfsDistance) -> Point -> BfsDistance
+                 -> (Point, BfsDistance)
+      minHideout (pMin, distMin) p dist =
+        if dist > minKnownBfs && dist < distMin
+           && Tile.isHideout coTileSpeedup (lvl `at` p)
+        then (p, dist)
+        else (pMin, distMin)
+      (p1, dist1) = PointArray.ifoldlA' minHideout (bpos b, maxBfsDistance) bfs
+  return $! if p1 == bpos b  -- possibly hideout underfoot; ignore
+            then Nothing
+            else Just (p1, subtractBfsDistance dist1 apartBfs)
