@@ -5,7 +5,8 @@ module Game.LambdaHack.Client.Preferences
     -- * Internal operations
   , effectToBenefit
   , averageTurnValue, avgItemDelay, avgItemLife, durabilityMult
-  , organBenefit, recBenefit, fakeItem, aspectToBenefit, aspectRecordToBenefit
+  , organBenefit, recBenefit, fakeItem
+  , aspectToBenefit, capStat, aspectRecordToBenefit
 #endif
   ) where
 
@@ -22,9 +23,9 @@ import           Game.LambdaHack.Common.Kind
 import           Game.LambdaHack.Common.Misc
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Types
+import           Game.LambdaHack.Content.FactionKind
 import           Game.LambdaHack.Content.ItemKind (ItemKind)
 import qualified Game.LambdaHack.Content.ItemKind as IK
-import           Game.LambdaHack.Content.ModeKind
 import qualified Game.LambdaHack.Core.Dice as Dice
 import qualified Game.LambdaHack.Definition.Ability as Ability
 import           Game.LambdaHack.Definition.Defs
@@ -32,7 +33,7 @@ import           Game.LambdaHack.Definition.Flavour
 
 -- | How much AI benefits from applying the effect.
 -- The first component is benefit when applied to self, the second
--- is benefit (preferably negative) when applied to enemy.
+-- is benefit (preferably negative) when applied to enemy (via melee).
 -- This represents benefit from using the effect every @avgItemDelay@ turns,
 -- so if the item is not durable, the value is adjusted down elsewhere.
 -- The benefit includes the drawback of having to use the actor's turn,
@@ -47,10 +48,10 @@ effectToBenefit :: COps -> FactionId -> FactionDict -> IK.Effect
 effectToBenefit cops fid factionD eff =
   let delta x = (x, x)
   in case eff of
-    IK.Burn d -> delta $ -(min 1500 $ 15 * Dice.meanDice d)
-      -- often splash damage, armor doesn't block (but HurtMelee doesn't boost)
-    IK.Explode "single spark" -> delta (-1)  -- hardwired; probing and flavour
-    IK.Explode "fragrance" -> (1, -5)  -- hardwired; situational
+    IK.Burn d -> delta $ -(min 1000 $ 10 * Dice.meanDice d)
+    IK.Explode IK.S_SINGLE_SPARK -> delta (-1)  -- probing and flavour
+    IK.Explode IK.S_SPARK -> delta (-9)  -- small, to not affect weapon order
+    IK.Explode IK.S_FRAGRANCE -> (1, -5)  -- situational
     IK.Explode _ ->
       -- There is a risk the explosion is focused and harmful to self
       -- or not focused and beneficial to nearby foes, but not to self.
@@ -59,24 +60,38 @@ effectToBenefit cops fid factionD eff =
       -- Due to this assumption, healing explosives should be wrapped
       -- in @OnSmash@, or else they are counted as an incentive for throwing
       -- an item at foes, which in that case is counterproductive.
-      delta (-100)
+      delta (-50)  -- not too low so that S_INK_SAC used by AI
     IK.RefillHP p ->
       delta $ if p > 0
-              then min 2000 (20 * fromIntegral p)
-              else max (-1000) (10 * fromIntegral p)
-        -- one HP healed is worth a bit more than one HP dealed to enemy,
+              then min 2000 (20 * intToDouble p)
+              else max (-1000) (10 * intToDouble p)
+        -- one HP healed is worth a bit more than one HP dealt to enemy,
         -- because if the actor survives, he may deal damage many times;
         -- however, AI is mostly for non-heroes that fight in suicidal crowds,
         -- so the two values are kept close enough to maintain berserk approach
-    IK.RefillCalm p -> delta $ if p > 0
-                               then min 100 (fromIntegral p)
-                               else max (-500) (5 * fromIntegral p)
+    IK.RefillCalm p ->
+      ( if p > 0
+        then min 100 (intToDouble p)
+          -- this may cause ice to be attractive to AI,
+          -- but it doesn't trigger it due to no @ConsideredByAI@
+        else if p >= -5
+             then max (-100) (intToDouble p)
+             else max (-1500) (15 * intToDouble p)
+          -- big Calm drains are incredibly dangerous, so don't be stupid
+          -- and don't self-inflict them, particularly if you are an intelligent
+          -- high-HP actor, which is likely if you collect and apply items
+      , if p > 0
+        then min 100 (intToDouble p)
+        else max (-500) (5 * intToDouble p) )
+          -- quite a powerful weapon, especially against high-HP foes
     IK.Dominate -> (0, -100)  -- I obtained an actor with, say 10HP,
                               -- worth 200, and enemy lost him, another 100;
                               -- divided by 3, because impression needed first
-    IK.Impress -> (0, -20)
-    IK.PutToSleep -> (10, -50)  -- can affect friends, but more often enemies
-    IK.Yell -> (-5, -10)  -- usually uncontrollably wakes up enemies, so bad
+    IK.Impress -> (0, -20)  -- this causes heroes to waste a crucial resource
+                            -- but makes aliens more aggresive than defensive;
+                            -- also, smart use is hardcoded in AI action choice
+    IK.PutToSleep -> (-10, -50)  -- can affect friends, but more often enemies
+    IK.Yell -> (-1, -2)  -- usually uncontrollably wakes up enemies, so bad
     IK.Summon grp d ->  -- contrived by not checking if enemies also control
                         -- that group; safe for normal dungeon crawl content;
                         -- not correct for symmetric scenarios, but let it be
@@ -84,16 +99,16 @@ effectToBenefit cops fid factionD eff =
           fact = factionD EM.! fid
           friendlyHasGrp fid2 =
             isFriend fid fact fid2
-            && grp `elem` fgroups (gplayer $ factionD EM.! fid2)
+            && fromMaybe 0 (lookup grp $ fgroups $ gkind $ factionD EM.! fid2)
+               > 0
       in -- Prefer applying summoning items to flinging them; the actor gets
-         -- spawned further from for, but it's more robust
+         -- spawned further from foes, but it's more robust.
          if any friendlyHasGrp $ EM.keys factionD
          then (ben, -1)
          else (-ben * 3, 1)  -- the foe may spawn during battle and gang up
-    IK.Ascend{} -> (-99, 99)
-      -- note the reversed values: only change levels sensibly, in teams,
-      -- and don't remove enemy too far, he may be easy to kill
-      -- and may have loot
+    IK.Ascend{} -> (0, 0)
+      -- only change levels sensibly, in teams, and don't remove enemy too far,
+      -- he may be easy to kill and may have essential loot
     IK.Escape{} -> (-9999, 9999)  -- even if can escape, loots first and then
                                   -- handles escape as a special case
     -- The following two are expensive, because they ofen activate
@@ -105,32 +120,34 @@ effectToBenefit cops fid factionD eff =
     IK.Paralyze d -> delta $ -20 * Dice.meanDice d  -- clips
     IK.ParalyzeInWater d -> delta $ -10 * Dice.meanDice d  -- clips; resistable
     IK.InsertMove d -> delta $ 10 * Dice.meanDice d  -- turns
-    IK.Teleport d -> if Dice.meanDice d <= 8
-                     then (0, 0)    -- annoying either way
-                     else (-9, -1)  -- for self, don't derail exploration
-                                    -- for foes, fight with one less at a time
-    IK.CreateItem COrgan "condition" _ ->
+    IK.Teleport{} -> (-9, -1)  -- for self, don't derail exploration
+                               -- for foes, fight with one less at a time
+    IK.CreateItem _ COrgan IK.CONDITION _ ->
       (1, -1)  -- varied, big bunch, but try to create it anyway
-    IK.CreateItem COrgan grp timer ->  -- assumed temporary
+    IK.CreateItem _ COrgan grp timer ->  -- assumed temporary
       let turnTimer = IK.foldTimer 1 Dice.meanDice Dice.meanDice timer
             -- copy count used instead of timer for organs with many copies
           (total, count) = organBenefit turnTimer grp cops fid factionD
-      in delta $ total / fromIntegral count
+      in delta $ total / intToDouble count
            -- the same when created in me and in foe
            -- average over all matching grps; simplified: rarities ignored
-    IK.CreateItem _ "treasure" _ -> (100, 0)  -- assumed not temporary
-    IK.CreateItem _ "common item" _ -> (70, 0)
-    IK.CreateItem _ "curious item" _ -> (70, 0)
-    IK.CreateItem _ "any scroll" _ -> (50, 0)
-    IK.CreateItem _ "any vial" _ -> (50, 0)
-    IK.CreateItem _ "potion" _ -> (50, 0)
-    IK.CreateItem _ "explosive" _ -> (50, 0)
-    IK.CreateItem _ "any jewelry" _ -> (100, 0)
-    IK.CreateItem _ grp _ ->  -- assumed not temporary and @grp@ tiny
+    IK.CreateItem _ _ IK.TREASURE _ -> (100, 0)  -- assumed not temporary
+    IK.CreateItem _ _ IK.COMMON_ITEM _ -> (70, 0)
+    IK.CreateItem _ _ IK.CRAWL_ITEM _ -> (70, 0)
+    IK.CreateItem _ _ IK.ANY_SCROLL _ -> (50, 0)
+    IK.CreateItem _ _ IK.ANY_GLASS _ -> (75, 0)
+    IK.CreateItem _ _ IK.ANY_POTION _ -> (100, 0)
+    IK.CreateItem _ _ IK.ANY_FLASK _ -> (50, 0)
+    IK.CreateItem _ _ IK.EXPLOSIVE _ -> (50, 0)
+    IK.CreateItem _ _ IK.ANY_JEWELRY _ -> (100, 0)
+    IK.CreateItem _ _ grp _ ->  -- assumed not temporary and @grp@ tiny
       let (total, count) = recBenefit grp cops fid factionD
-      in (total / fromIntegral count, 0)
-    IK.DropItem _ _ COrgan "condition" ->
-      delta 30  -- save for curing own bad conditions
+      in (total / intToDouble count, 0)
+    IK.DestroyItem{} -> delta (-10)  -- potentially harmful
+    IK.ConsumeItems{} -> delta (-10)  -- potentially harmful
+    IK.DropItem _ _ COrgan IK.CONDITION ->
+      (0, -1)  -- negative value necessary to collect such items;
+               -- smart use on self is hardcoded in AI action choice
     IK.DropItem ngroup kcopy COrgan grp ->  -- assumed temporary
       -- Simplified: we assume actor has an average number of copies
       -- (and none have yet run out, e.g., prompt curing of poisoning)
@@ -142,39 +159,76 @@ effectToBenefit cops fid factionD eff =
           (total, count) = organBenefit turnTimer grp cops fid factionD
           boundBonus n = if n == maxBound then 10 else 0
       in delta $ boundBonus ngroup + boundBonus kcopy
-                 - total / fromIntegral count
+                 - total / intToDouble count
                    -- the same when dropped from me and foe
     IK.DropItem{} -> delta (-10)  -- depends a lot on what is dropped
+    IK.Recharge n d -> delta $ intToDouble n * Dice.meanDice d / 10
+      -- this high value to price weapons with @OnUser@ over fists
+    IK.Discharge n d -> delta $ - intToDouble n * Dice.meanDice d / 10
     IK.PolyItem -> (1, 0)  -- may fizzle, so AI never uses (could loop)
     IK.RerollItem -> (1, 0)  -- may fizzle, so AI never uses (could loop)
     IK.DupItem -> (1, 0)  -- may fizzle, so AI never uses (could loop)
     IK.Identify -> (1, 0)  -- may fizzle, so AI never uses (could loop)
-    IK.Detect IK.DetectAll radius -> (fromIntegral radius * 2, 0)
-    IK.Detect IK.DetectLoot radius -> (fromIntegral radius * 2, 0)
-    IK.Detect IK.DetectExit radius -> (fromIntegral radius / 2, 0)
-    IK.Detect _ radius -> (fromIntegral radius, 0)
-    IK.SendFlying _ -> (0, -100)  -- very context dependent, but lack of control
-    IK.PushActor _ -> (0, -100)   -- is deadly on some maps, leading to harm;
-    IK.PullActor _ -> (0, -100)   -- pushing others may crush them against wall
-    IK.DropBestWeapon -> delta $ -50  -- often a whole turn wasted == InsertMove
-    IK.ActivateInv ' ' -> delta $ -200  -- brutal and deadly
-    IK.ActivateInv _ -> delta $ -50  -- depends on the items
+    IK.Detect IK.DetectAll radius -> (intToDouble radius * 2, 0)
+    IK.Detect IK.DetectLoot radius -> (intToDouble radius * 2, 0)
+    IK.Detect IK.DetectExit radius -> (intToDouble radius / 2, 0)
+    IK.Detect _ radius -> (intToDouble radius, 0)
+    IK.SendFlying _ -> (0, -1)   -- very context dependent, but lack of control
+    IK.PullActor _ -> (0, -1)    -- is deadly on some maps, leading to harm;
+    IK.PushActor _ -> (0, -100)  -- pushing others may crush them against wall
+                                 -- and give us time to fling at them
     IK.ApplyPerfume -> delta 0  -- depends on smell sense of friends and foes
-    IK.OneOf efs ->
-      let bs = map (effectToBenefit cops fid factionD) efs
+    IK.AtMostOneOf effs ->
+      let bs = map (effectToBenefit cops fid factionD) effs
           f (self, foe) (accSelf, accFoe) = (self + accSelf, foe + accFoe)
           (effSelf, effFoe) = foldr f (0, 0) bs
-      in (effSelf / fromIntegral (length bs), effFoe / fromIntegral (length bs))
+      in (effSelf / intToDouble (length bs), effFoe / intToDouble (length bs))
+    IK.OneOf effs ->
+      let bs = map (effectToBenefit cops fid factionD) effs
+          f (self, foe) (accSelf, accFoe) = (self + accSelf, foe + accFoe)
+          (effSelf, effFoe) = foldr f (0, 0) bs
+      in (effSelf / intToDouble (length bs), effFoe / intToDouble (length bs))
     IK.OnSmash _ -> delta 0
       -- can be beneficial; we'd need to analyze explosions, range, etc.
-    IK.VerbNoLonger{} -> delta 0  -- flavour only, no benefit
-    IK.VerbMsg{} -> delta 0  -- flavour only, no benefit
-    IK.Composite [] -> delta 0
-    IK.Composite (eff1 : _) -> effectToBenefit cops fid factionD eff1
+    IK.OnCombine eff1 -> effectToBenefit cops fid factionD eff1
+    IK.OnUser eff1 ->
+      let (effSelf, _) = effectToBenefit cops fid factionD eff1
+      in (effSelf, - effSelf)
+           -- in both cases just applies the effect to itself,
+           -- which is approximately equal to applying the opposite to foe;
+           -- this may result in double-counting, but ensures that weapons
+           -- that harm their wielders are properly discouted;
+           -- in a way, this should be double-counted, because the effect
+           -- not only hinders (or enhances) applying the item,
+           -- but meleeing with it, too
+    IK.NopEffect -> delta 0
+    IK.AndEffect eff1 _ -> effectToBenefit cops fid factionD eff1
       -- for simplicity; so in content make sure to place initial animations
       -- among normal effects, not at the start of composite effect
       -- (animations should not fail, after all), and start composite
       -- effect with the main thing
+    IK.OrEffect eff1 _ -> effectToBenefit cops fid factionD eff1
+    IK.SeqEffect effs -> effectToBenefits cops fid factionD effs
+    IK.When _cond eff1 ->
+      -- Assuming the condition met most of the time. Really, too hard for AI.
+      effectToBenefit cops fid factionD eff1
+    IK.Unless _cond eff1 ->
+      -- Assuming the condition *not* met most of the time.
+      -- Really, too hard for AI.
+      effectToBenefit cops fid factionD eff1
+    IK.IfThenElse _cond eff1 _eff2 ->
+      -- Assuming the first is much more common. Really, too hard for AI.
+      effectToBenefit cops fid factionD eff1
+    IK.VerbNoLonger{} -> delta 0  -- flavour only, no benefit
+    IK.VerbMsg{} -> delta 0  -- flavour only, no benefit
+    IK.VerbMsgFail{} -> delta 0
+
+effectToBenefits :: COps -> FactionId -> FactionDict -> [IK.Effect]
+                 -> (Double, Double)
+effectToBenefits cops fid factionD effs =
+  let effPairs = map (effectToBenefit cops fid factionD) effs
+      f (self, foe) (accSelf, accFoe) = (self + accSelf, foe + accFoe)
+  in foldr f (0, 0) effPairs
 
 -- See the comment for @Paralyze@.
 averageTurnValue :: Double
@@ -260,12 +314,12 @@ organBenefit turnTimer grp cops@COps{coitem} fid factionD =
   let f (!sacc, !pacc) !p _ !kind =
         let count = Dice.meanDice (IK.icount kind)
             paspect asp =
-              fromIntegral p
+              intToDouble p
               * count * turnTimer
                 -- the aspect stays for this many turns'
                * aspectToBenefit asp
             peffect eff =
-              fromIntegral p
+              intToDouble p
               * count
                 -- this many consecutive effects will be generated, if any
               * fst (effectToBenefit cops fid factionD eff)
@@ -294,7 +348,7 @@ fakeItem :: ContentId IK.ItemKind -> IK.ItemKind -> IA.KindMean -> ItemFull
 fakeItem kindId kind km =
   let jkind    = IdentityObvious kindId
       jfid     = Nothing  -- the default
-      jflavour = Flavour (toEnum 0) (toEnum 0) -- dummy
+      jflavour = dummyFlavour
       itemBase = Item{..}
       itemDisco = ItemDiscoMean km
   in ItemFull itemBase kindId kind itemDisco True
@@ -333,20 +387,20 @@ aspectToBenefit :: IK.Aspect -> Double
 aspectToBenefit asp =
   case asp of
     IK.Timeout{} -> 0
-    IK.AddSkill Ability.SkMove p -> Dice.meanDice p * 5
-    IK.AddSkill Ability.SkMelee p -> Dice.meanDice p * 5
-    IK.AddSkill Ability.SkDisplace p -> Dice.meanDice p * 5
-    IK.AddSkill Ability.SkAlter p -> Dice.meanDice p * 5
-    IK.AddSkill Ability.SkWait p -> Dice.meanDice p * 5
-    IK.AddSkill Ability.SkMoveItem p -> Dice.meanDice p * 5
-    IK.AddSkill Ability.SkProject p -> Dice.meanDice p * 5
-    IK.AddSkill Ability.SkApply p -> Dice.meanDice p * 5
+    IK.AddSkill Ability.SkMove p -> capStat (Dice.meanDice p) * 5
+    IK.AddSkill Ability.SkMelee p -> capStat (Dice.meanDice p) * 5
+    IK.AddSkill Ability.SkDisplace p -> capStat (Dice.meanDice p)
+    IK.AddSkill Ability.SkAlter p -> capStat (Dice.meanDice p)
+    IK.AddSkill Ability.SkWait p -> capStat (Dice.meanDice p)
+    IK.AddSkill Ability.SkMoveItem p -> capStat (Dice.meanDice p)
+    IK.AddSkill Ability.SkProject p -> capStat (Dice.meanDice p) * 2
+    IK.AddSkill Ability.SkApply p -> capStat (Dice.meanDice p) * 2
     IK.AddSkill Ability.SkSwimming p -> Dice.meanDice p
     IK.AddSkill Ability.SkFlying p -> Dice.meanDice p
     IK.AddSkill Ability.SkHurtMelee p -> Dice.meanDice p  -- offence favoured
     IK.AddSkill Ability.SkArmorMelee p -> Dice.meanDice p / 4
                                               -- only partial protection
-    IK.AddSkill Ability.SkArmorRanged p -> Dice.meanDice p / 8
+    IK.AddSkill Ability.SkArmorRanged p -> Dice.meanDice p / 4
     IK.AddSkill Ability.SkMaxHP p -> Dice.meanDice p
     IK.AddSkill Ability.SkMaxCalm p -> Dice.meanDice p / 5
     IK.AddSkill Ability.SkSpeed p -> Dice.meanDice p * 25
@@ -355,15 +409,20 @@ aspectToBenefit asp =
     IK.AddSkill Ability.SkSight p -> Dice.meanDice p * 5
     IK.AddSkill Ability.SkSmell p -> Dice.meanDice p
     IK.AddSkill Ability.SkShine p -> Dice.meanDice p * 2
-    IK.AddSkill Ability.SkNocto p -> Dice.meanDice p * 10
+    IK.AddSkill Ability.SkNocto p -> Dice.meanDice p * 30
                                        -- > sight + light; stealth, slots
     IK.AddSkill Ability.SkHearing p -> Dice.meanDice p
     IK.AddSkill Ability.SkAggression _ -> 0  -- dunno
-    IK.AddSkill Ability.SkOdor p -> - Dice.meanDice p  -- makes one trackable
+    IK.AddSkill Ability.SkOdor p -> - Dice.meanDice p / 4
+      -- rarely, if big enough, determines if one is trackable
+    IK.AddSkill Ability.SkDeflectRanged p -> Dice.meanDice p * 100
+    IK.AddSkill Ability.SkDeflectMelee p -> Dice.meanDice p * 100
     IK.SetFlag{} -> 0
+      -- valuing @UnderRanged@ and @UnderMelee@ vs retaining the charge
+      -- and explicit applying is too hard, hence ignored
     IK.ELabel{} -> 0
     IK.ToThrow{} -> 0  -- counted elsewhere
-    IK.HideAs{} -> 0
+    IK.PresentAs{} -> 0
     IK.EqpSlot{} -> 0
     IK.Odds{} -> 0
       -- Should be already rolled; if not, can't tell easily.
@@ -372,6 +431,11 @@ aspectToBenefit asp =
       -- in not fully-identified items, because they are so crucial
       -- for evaluation.
 
+-- We simplify, assuming stats are unlikely to be higher than 10
+-- and to be affected by more than one non-organ item at a time.
+capStat :: Double -> Double
+capStat x = max (-10) $ min 10 x
+
 aspectRecordToBenefit :: IA.AspectRecord -> [Double]
 aspectRecordToBenefit arItem =
   map aspectToBenefit $ IA.aspectRecordToList arItem
@@ -385,7 +449,7 @@ totalUsefulness cops fid factionD itemFull@ItemFull{itemKind, itemSuspect} =
       -- because we assume it's in equipment and then
       -- we can't effectively apply it, because it's never recharged,
       -- because it activates as soon as recharged.
-      -- We ignore the rare case of a periodic item kept in backpack
+      -- We ignore the rare case of a periodic item kept in stash
       -- to be applied manually. AI is too silly to choose it and we
       -- certainly don't want AI to destroy periodic items out of silliness.
       -- We don't assign a special bonus or malus due to being periodic,
@@ -407,7 +471,7 @@ totalUsefulness cops fid factionD itemFull@ItemFull{itemKind, itemSuspect} =
       -- Timeout 2 means two such items are needed to use the effect each turn,
       -- so a single such item may be worth half of the permanent value.
       -- E.g., when item heals 1 HP each turn, that's precisly the calculation.
-      timeout = fromIntegral $ IA.aTimeout arItem
+      timeout = intToDouble $ IA.aTimeout arItem
       scalePeriodic value = value / max 1 timeout
       -- With non-periodic items, when we need to expend a turn to apply the
       -- item or, e.g., we lose the opportunity to use another weapon if we hit
@@ -421,10 +485,7 @@ totalUsefulness cops fid factionD itemFull@ItemFull{itemKind, itemSuspect} =
       timeoutSqrt = sqrt $ max 2 timeout
       scaleTimeout v = v / timeoutSqrt
       (effSelf, effFoe) =
-        let effPairs = map (effectToBenefit cops fid factionD)
-                           (IK.ieffects itemKind)
-            f (self, foe) (accSelf, accFoe) = (self + accSelf, foe + accFoe)
-        in foldr f (0, 0) effPairs
+        effectToBenefits cops fid factionD (IK.ieffects itemKind)
       -- Durability doesn't have any numerical impact on @eqpSum,
       -- because item is never consumed by just being stored in equipment.
       -- Also no numerical impact for flinging, because we can't fling it again
@@ -446,6 +507,14 @@ totalUsefulness cops fid factionD itemFull@ItemFull{itemKind, itemSuspect} =
         else scaleTimeout (effSelf + effDice)
                -- hits self with kintetic dice too, when applying
              / if durable then 1 else durabilityMult
+      -- This assumes attacker hurt skill and enemy armor skill balance
+      -- and so this value doesn't need to be recomputed at each equipment
+      -- change and distributing weapons among AI actors doesn't need
+      -- to match each weapon to each actor's equipment. However,
+      -- a bad side-effect is that if an actor has terrible hurt skill,
+      -- a weapon with high dice is still used by him before a burning weapon.
+      -- Unless the opponent has even more terrible armor, unlikely,
+      -- the chosen weapon is definitely not the best.
       effDice = - IK.damageUsefulness itemKind
       -- For melee, we add the foe part only.
       benMelee = if periodic
@@ -475,11 +544,11 @@ totalUsefulness cops fid factionD itemFull@ItemFull{itemKind, itemSuspect} =
         hurtMult = armorHurtCalculation True (IA.aSkills arItem)
                                              Ability.zeroSkills
         dmg = Dice.meanDice $ IK.idamage itemKind
-        rawDeltaHP = ceiling $ fromIntegral hurtMult * xD dmg / 100
+        rawDeltaHP = ceiling $ intToDouble hurtMult * xD dmg / 100
         -- For simplicity, we ignore range bonus/malus and @Lobable@.
         IK.ThrowMod{IK.throwVelocity} = IA.aToThrow arItem
         speed = speedFromWeight (IK.iweight itemKind) throwVelocity
-        v = - fromIntegral (modifyDamageBySpeed rawDeltaHP speed) * 10 / xD 1
+        v = - int64ToDouble (modifyDamageBySpeed rawDeltaHP speed) * 10 / xD 1
           -- 1 damage valued at 10, just as in @damageUsefulness@
       -- If item is periodic, we factor in the self value of effects,
       -- because they are applied to self, whether the actor wants it or not.
@@ -498,8 +567,9 @@ totalUsefulness cops fid factionD itemFull@ItemFull{itemKind, itemSuspect} =
       -- value those competently against any equally enormous bonuses
       -- the item might provide to compensate and so be even considered.
       cripplingDrawback = not (null aspectBenefits)
-                          && minimum aspectBenefits < -20
+                          && minimum aspectBenefits < -25
       eqpSum = eqpBens - if cripplingDrawback then 100 else 0
+      vApplyFling = max benApply (- benFling)
       -- If a weapon heals enemy at impact, given choice, it won't be used
       -- for melee, but can be equipped anyway, for beneficial aspects.
       -- OTOH, cif it harms wearer too much, it won't be worn
@@ -509,19 +579,31 @@ totalUsefulness cops fid factionD itemFull@ItemFull{itemKind, itemSuspect} =
             -- the flag probably known even if item not identified
           && (benMelee < 0 || itemSuspect)
           && eqpSum >= -20 =
-          ( True  -- equip, melee crucial and only weapons in eqp can be used
-          , eqpSum
-            + maximum [benApply, - benMeleeAvg, 0] )  -- apply or melee or not
+            let vEqp = eqpSum + maximum [benApply, - benMeleeAvg, 0]
+                      -- equip plus apply or melee or not
+                v = if | vEqp > 0 -> vEqp
+                           -- pick up to equip; melee is crucial
+                       | vApplyFling > 0 -> vApplyFling
+                           -- at least pick up to apply or fling, if feasible,
+                           -- and equip just in case interesting effect;
+                           -- will be taken off if very harmful
+                       | otherwise -> vEqp
+                           -- do not pick up, but if forced, the best bet
+                           -- is equip anyway
+            in (True, v)
         | (IA.goesIntoEqp arItem
-          || isJust (lookup "condition" $ IK.ifreq itemKind))
-               -- hack to record benefit, to use it in calculations later on
+           || IA.checkFlag Ability.Condition arItem)
+                -- hack to record benefit, to use, e.g., to assign colour
           && (eqpSum > 0 || itemSuspect) =  -- weapon or other equippable
           ( True  -- equip; long time bonus usually outweighs fling or apply
           , eqpSum  -- possibly spent turn equipping, so reap the benefits
             + if durable
               then benApply  -- apply or not but don't fling
               else 0)  -- don't remove from equipment by using up
-        | otherwise =
-          (False, max benApply (- benFling))  -- apply or fling
-      benPickup = max benPickupRaw $ if itemSuspect then 10 else 0
+        | otherwise = (False, vApplyFling)  -- apply or fling
+      benPickupRaw2 = max benPickupRaw $ if itemSuspect then 10 else 0
+      -- If periodic, pick up to deny to foes and sometimes to apply
+      -- to activate the first effect only (easier than computing if the first
+      -- effect is really beneficial, while all effects detrimental).
+      benPickup = if periodic then max 1 benPickupRaw2 else benPickupRaw2
   in Benefit{..}
