@@ -518,7 +518,7 @@ alterCommon leader bumping tpos = do
                map (\(iid, kit) -> (getKind iid, (iid, kit))) (EM.assocs embeds)
              ilooks = map itemLook $ sortEmbeds cops t embedKindList
          failWith $ makePhrase $
-           ["there is no point kicking", MU.AW name]
+           ["there is no way to activate or modify", MU.AW name]
            ++ if EM.null embeds
               then []
               else ["with", MU.WWandW ilooks]
