@@ -8,14 +8,14 @@ module Game.LambdaHack.Client.UI.Key
   , upKM, downKM, leftKM, rightKM
   , homeKM, endKM, backspaceKM, controlP
   , leftButtonReleaseKM, middleButtonReleaseKM, rightButtonReleaseKM
-  , dirAllKey, handleDir, moveBinding, mkKM, mkChar, mkKP
-  , keyTranslate, keyTranslateWeb
+  , cardinalAllKM, dirAllKey, handleCardinal, handleDir, moveBinding
+  , mkKM, mkChar, keyTranslate, keyTranslateWeb
+  , dirMoveNoModifier, dirRunNoModifier, dirRunControl, dirRunShift
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
   , dirKeypadKey, dirKeypadShiftChar, dirKeypadShiftKey
-  , dirLaptopKey, dirLaptopShiftKey
+  , dirLeftHandKey, dirLeftHandShiftKey
   , dirViChar, dirViKey, dirViShiftKey
-  , dirMoveNoModifier, dirRunNoModifier, dirRunControl, dirRunShift
 #endif
   ) where
 
@@ -28,7 +28,7 @@ import           Data.Binary
 import qualified Data.Char as Char
 import           GHC.Generics (Generic)
 
-import Game.LambdaHack.Common.Point
+import Game.LambdaHack.Client.UI.PointUI
 import Game.LambdaHack.Common.Vector
 
 -- | Frontend-independent datatype to represent keys.
@@ -74,6 +74,7 @@ instance NFData Key
 data Modifier =
     NoModifier
   | ControlShift
+  | AltShift
   | Shift
   | Control
   | Alt
@@ -97,7 +98,7 @@ instance Show KM where
 
 -- | Key, modifier and position of mouse pointer.
 data KMP = KMP { kmpKeyMod  :: KM
-               , kmpPointer :: Point }
+               , kmpPointer :: PointUI }
 
 -- | Common and terse names for keys.
 showKey :: Key -> String
@@ -130,13 +131,14 @@ showKey MiddleButtonRelease = "MMB"
 showKey RightButtonRelease = "RMB"
 showKey WheelNorth = "WHEEL-UP"
 showKey WheelSouth = "WHEEL-DN"
-showKey (Unknown s) = "'" ++ s ++ "'"
+showKey (Unknown s) = s
 showKey DeadKey      = "DEADKEY"
 
 -- | Show a key with a modifier, if any.
 showKM :: KM -> String
 showKM KM{modifier=NoModifier, key} = showKey key
 showKM KM{modifier=ControlShift, key} = "C-S-" ++ showKey key
+showKM KM{modifier=AltShift, key} = "A-S-" ++ showKey key
 showKM KM{modifier=Shift, key} = "S-" ++ showKey key
 showKM KM{modifier=Control, key} = "C-" ++ showKey key
 showKM KM{modifier=Alt, key} = "A-" ++ showKey key
@@ -204,6 +206,9 @@ middleButtonReleaseKM = KM NoModifier MiddleButtonRelease
 rightButtonReleaseKM :: KM
 rightButtonReleaseKM = KM NoModifier RightButtonRelease
 
+cardinalKeypadKM :: [KM]
+cardinalKeypadKM = map (KM NoModifier) [Up, Right, Down, Left]
+
 dirKeypadKey :: [Key]
 dirKeypadKey = [Home, Up, PgUp, Right, PgDn, Down, End, Left]
 
@@ -213,11 +218,17 @@ dirKeypadShiftChar = ['7', '8', '9', '6', '3', '2', '1', '4']
 dirKeypadShiftKey :: [Key]
 dirKeypadShiftKey = map KP dirKeypadShiftChar
 
-dirLaptopKey :: [Key]
-dirLaptopKey = map Char ['7', '8', '9', 'o', 'l', 'k', 'j', 'u']
+cardinalLeftHandKM :: [KM]
+cardinalLeftHandKM = map (KM NoModifier . Char) ['w', 'd', 'x', 'a']
 
-dirLaptopShiftKey :: [Key]
-dirLaptopShiftKey = map Char ['&', '*', '(', 'O', 'L', 'K', 'J', 'U']
+dirLeftHandKey :: [Key]
+dirLeftHandKey = map Char ['q', 'w', 'e', 'd', 'c', 'x', 'z', 'a']
+
+dirLeftHandShiftKey :: [Key]
+dirLeftHandShiftKey = map Char ['Q', 'W', 'E', 'D', 'C', 'X', 'Z', 'A']
+
+cardinalViKM :: [KM]
+cardinalViKM = map (KM NoModifier . Char) ['k', 'l', 'j', 'h']
 
 dirViChar :: [Char]
 dirViChar = ['y', 'k', 'u', 'l', 'n', 'j', 'b', 'h']
@@ -229,16 +240,14 @@ dirViShiftKey :: [Key]
 dirViShiftKey = map (Char . Char.toUpper) dirViChar
 
 dirMoveNoModifier :: Bool -> Bool -> [Key]
-dirMoveNoModifier uVi uLaptop =
-  dirKeypadKey ++ if | uVi -> dirViKey
-                     | uLaptop -> dirLaptopKey
-                     | otherwise -> []
+dirMoveNoModifier uVi uLeftHand =
+  dirKeypadKey ++ (if uVi then dirViKey else [])
+               ++ (if uLeftHand then dirLeftHandKey else [])
 
 dirRunNoModifier :: Bool -> Bool -> [Key]
-dirRunNoModifier uVi uLaptop =
-  dirKeypadShiftKey ++ if | uVi -> dirViShiftKey
-                          | uLaptop -> dirLaptopShiftKey
-                          | otherwise -> []
+dirRunNoModifier uVi uLeftHand =
+  dirKeypadShiftKey ++ (if uVi then dirViShiftKey else [])
+                    ++ (if uLeftHand then dirLeftHandShiftKey else [])
 
 dirRunControl :: [Key]
 dirRunControl = dirKeypadKey
@@ -248,31 +257,42 @@ dirRunControl = dirKeypadKey
 dirRunShift :: [Key]
 dirRunShift = dirRunControl
 
+cardinalAllKM :: Bool -> Bool -> [KM]
+cardinalAllKM uVi uLeftHand = concat $
+  [cardinalKeypadKM]
+  ++ [cardinalViKM | uVi]
+  ++ [cardinalLeftHandKM | uLeftHand]
+
 dirAllKey :: Bool -> Bool -> [Key]
-dirAllKey uVi uLaptop =
-  dirMoveNoModifier uVi uLaptop
-  ++ dirRunNoModifier uVi uLaptop
+dirAllKey uVi uLeftHand =
+  dirMoveNoModifier uVi uLeftHand
+  ++ dirRunNoModifier uVi uLeftHand
   ++ dirRunControl
 
+handleCardinal :: [KM] -> KM -> Maybe Vector
+handleCardinal dirKeys key =
+  let assocs = zip dirKeys $ cycle movesCardinal
+  in lookup key assocs
+
 -- | Configurable event handler for the direction keys.
 -- Used for directed commands such as close door.
-handleDir :: Bool -> Bool -> KM -> Maybe Vector
-handleDir uVi uLaptop KM{modifier=NoModifier, key} =
-  let assocs = zip (dirAllKey uVi uLaptop) $ cycle moves
+handleDir :: [Key] -> KM -> Maybe Vector
+handleDir dirKeys KM{modifier=NoModifier, key} =
+  let assocs = zip dirKeys $ cycle moves
   in lookup key assocs
-handleDir _ _ _ = Nothing
+handleDir _ _ = Nothing
 
 -- | Binding of both sets of movement keys, vi and laptop.
 moveBinding :: Bool -> Bool -> (Vector -> a) -> (Vector -> a)
             -> [(KM, a)]
-moveBinding uVi uLaptop move run =
-  let assign f (km, dir) = (km, f dir)
+moveBinding uVi uLeftHand move run =
+  let assign f km dir = (km, f dir)
       mapMove modifier keys =
-        map (assign move) (zip (map (KM modifier) keys) $ cycle moves)
+        zipWith (assign move) (map (KM modifier) keys) (cycle moves)
       mapRun modifier keys =
-        map (assign run) (zip (map (KM modifier) keys) $ cycle moves)
-  in mapMove NoModifier (dirMoveNoModifier uVi uLaptop)
-     ++ mapRun NoModifier (dirRunNoModifier uVi uLaptop)
+        zipWith (assign run) (map (KM modifier) keys) (cycle moves)
+  in mapMove NoModifier (dirMoveNoModifier uVi uLeftHand)
+     ++ mapRun NoModifier (dirRunNoModifier uVi uLeftHand)
      ++ mapRun Control dirRunControl
      ++ mapRun Shift dirRunShift
 
@@ -284,6 +304,8 @@ mkKM s = let mkKey sk =
          in case s of
            'C':'-':'S':'-':rest -> KM ControlShift (mkKey rest)
            'S':'-':'C':'-':rest -> KM ControlShift (mkKey rest)
+           'A':'-':'S':'-':rest -> KM AltShift (mkKey rest)
+           'S':'-':'A':'-':rest -> KM AltShift (mkKey rest)
            'S':'-':rest -> KM Shift (mkKey rest)
            'C':'-':rest -> KM Control (mkKey rest)
            'A':'-':rest -> KM Alt (mkKey rest)
@@ -292,9 +314,6 @@ mkKM s = let mkKey sk =
 mkChar :: Char -> KM
 mkChar c = KM NoModifier $ Char c
 
-mkKP :: Char -> KM
-mkKP c = KM NoModifier $ KP c
-
 -- | Translate key from a GTK string description to our internal key type.
 -- To be used, in particular, for the command bindings and macros
 -- in the config file.
@@ -312,10 +331,10 @@ keyTranslate "numbersign"    = Char '#'
 keyTranslate "dollar"        = Char '$'
 keyTranslate "parenleft"     = Char '('
 keyTranslate "parenright"    = Char ')'
-keyTranslate "asterisk"      = Char '*'  -- for latop movement keys
-keyTranslate "KP_Multiply"   = KP '*'    -- for keypad aiming
+keyTranslate "asterisk"      = Char '*'  -- KP and normal are merged here
+keyTranslate "KP_Multiply"   = Char '*'
 keyTranslate "slash"         = Char '/'
-keyTranslate "KP_Divide"     = KP '/'
+keyTranslate "KP_Divide"     = Char '/'
 keyTranslate "bar"           = Char '|'
 keyTranslate "backslash"     = Char '\\'
 keyTranslate "asciicircum"   = Char '^'
@@ -496,13 +515,13 @@ keyTranslateWeb "End"        _ = End
 keyTranslateWeb "Insert"     _ = Insert
 keyTranslateWeb "space"      _ = Space
 keyTranslateWeb "Equals"     _ = Char '='
-keyTranslateWeb "Multiply"   True = Char '*'  -- for laptop movement keys
-keyTranslateWeb "Multiply"   False = KP '*'     -- for keypad aiming
-keyTranslateWeb "*"          False = KP '*'     -- for keypad aiming
+keyTranslateWeb "Multiply"   _ = Char '*'  -- KP and normal are merged here
+keyTranslateWeb "*"          _ = Char '*'
 keyTranslateWeb "Add"        _ = Char '+'  -- KP and normal are merged here
 keyTranslateWeb "Subtract"   _ = Char '-'  -- KP and normal are merged here
 keyTranslateWeb "Divide"     True = Char '?'
 keyTranslateWeb "Divide"     False = Char '/' -- KP and normal are merged here
+keyTranslateWeb "/"          True = Char '?'
 keyTranslateWeb "/"          False = Char '/' -- KP and normal are merged here
 keyTranslateWeb "Decimal"    _ = Char '.'  -- dot and comma are merged here
 keyTranslateWeb "Separator"  _ = Char '.'  -- to sidestep national standards
