@@ -1,12 +1,13 @@
 -- | Server operations performed periodically in the game loop
 -- and related operations.
 module Game.LambdaHack.Server.PeriodicM
-  ( spawnMonster, addAnyActor
+  ( spawnMonster, addManyActors
   , advanceTime, advanceTimeTraj, overheadActorTime, swapTime
   , updateCalm, leadLevelSwitch
+  , endOrLoop
 #ifdef EXPOSE_INTERNAL
     -- * Internal operations
-  , rollSpawnPos
+  , addAnyActor, rollSpawnPos, gameExit
 #endif
   ) where
 
@@ -17,7 +18,8 @@ import Game.LambdaHack.Core.Prelude
 import qualified Data.EnumMap.Strict as EM
 import qualified Data.EnumSet as ES
 import           Data.Int (Int64)
-import           Data.Ord
+import qualified Data.IntMap.Strict as IM
+import qualified Data.Text as T
 
 import           Game.LambdaHack.Atomic
 import           Game.LambdaHack.Common.Actor
@@ -36,6 +38,7 @@ import qualified Game.LambdaHack.Common.Tile as Tile
 import           Game.LambdaHack.Common.Time
 import           Game.LambdaHack.Common.Types
 import qualified Game.LambdaHack.Content.CaveKind as CK
+import           Game.LambdaHack.Content.FactionKind
 import           Game.LambdaHack.Content.ItemKind (ItemKind)
 import qualified Game.LambdaHack.Content.ItemKind as IK
 import           Game.LambdaHack.Content.ModeKind
@@ -45,19 +48,23 @@ import qualified Game.LambdaHack.Definition.Ability as Ability
 import           Game.LambdaHack.Definition.Defs
 import           Game.LambdaHack.Server.CommonM
 import           Game.LambdaHack.Server.ItemM
+import           Game.LambdaHack.Server.ItemRev
 import           Game.LambdaHack.Server.MonadServer
+import           Game.LambdaHack.Server.ProtocolM
+import           Game.LambdaHack.Server.ServerOptions
 import           Game.LambdaHack.Server.State
 
 -- | Spawn, possibly, a monster according to the level's actor groups.
 -- We assume heroes are never spawned.
 spawnMonster :: MonadServerAtomic m => m ()
 spawnMonster = do
-  COps{cocave} <- getsState scops
-  arenas <- getsServer sarenas
+ COps{cocave} <- getsState scops
+ arenas <- getsServer sarenas
+ unless (ES.null arenas) $ do
   -- Do this on only one of the arenas to prevent micromanagement,
   -- e.g., spreading leaders across levels to bump monster generation.
-  arena <- rndToAction $ oneOf arenas
-  Level{lkind, ldepth, lbig} <- getLevel arena
+  arena <- rndToAction $ oneOf $ ES.elems arenas
+  Level{lkind, ldepth, lbig, ltime=localTime} <- getLevel arena
   let ck = okind cocave lkind
   if | CK.cactorCoeff ck == 0 || null (CK.cactorFreq ck) -> return ()
      | EM.size lbig >= 300 ->  -- probably not so rare, but debug anyway
@@ -67,62 +74,105 @@ spawnMonster = do
      | otherwise -> do
        totalDepth <- getsState stotalDepth
        lvlSpawned <- getsServer $ fromMaybe 0 . EM.lookup arena . snumSpawned
-       rc <- rndToAction
-             $ monsterGenChance ldepth totalDepth lvlSpawned (CK.cactorCoeff ck)
-       when rc $ do
+       let perMillion =
+             monsterGenChance ldepth totalDepth lvlSpawned (CK.cactorCoeff ck)
+           million = 1000000
+       k <- rndToAction $ randomR (1, million)
+       when (k <= perMillion && localTime > timeTurn) $ do
+         let numToSpawn | 25 * k <= perMillion = 3
+                        | 10 * k <= perMillion = 2
+                        | otherwise = 1
+             alt Nothing = Just 1
+             alt (Just n) = Just $ n + 1
          modifyServer $ \ser ->
-           ser {snumSpawned = EM.insert arena (lvlSpawned + 1)
-                              $ snumSpawned ser}
-         localTime <- getsState $ getLocalTime arena
-         maid <- addAnyActor False lvlSpawned (CK.cactorFreq ck) arena
-                             localTime Nothing
-         case maid of
-           Nothing -> return ()  -- suspect content; server debug elsewhere
-           Just aid -> do
-             b <- getsState $ getActorBody aid
-             mleader <- getsState $ gleader . (EM.! bfid b) . sfactionD
-             when (isNothing mleader) $ setFreshLeader (bfid b) aid
+           ser { snumSpawned = EM.insert arena (lvlSpawned + numToSpawn)
+                               $ snumSpawned ser
+               , sbandSpawned = IM.alter alt numToSpawn
+                                $ sbandSpawned ser }
+         void $ addManyActors False lvlSpawned (CK.cactorFreq ck) arena
+                              localTime Nothing numToSpawn
 
 addAnyActor :: MonadServerAtomic m
             => Bool -> Int -> Freqs ItemKind -> LevelId -> Time -> Maybe Point
-            -> m (Maybe ActorId)
+            -> m (Maybe (ActorId, Point))
 addAnyActor summoned lvlSpawned actorFreq lid time mpos = do
   -- We bootstrap the actor by first creating the trunk of the actor's body
   -- that contains the fixed properties of all actors of that kind.
   cops <- getsState scops
-  lvl <- getLevel lid
+  lvl@Level{ldepth} <- getLevel lid
   factionD <- getsState sfactionD
-  freq <- prepareItemKind lvlSpawned lid actorFreq
-  m2 <- rollItemAspect freq lid
+  freq <- prepareItemKind lvlSpawned ldepth actorFreq
+  m2 <- rollItemAspect freq ldepth
   case m2 of
-    Nothing -> do
-      debugPossiblyPrint "Server: addAnyActor: trunk failed to roll"
+    NoNewItem -> do
+      debugPossiblyPrint $ T.pack $
+        "Server: addAnyActor: trunk failed to roll"
+        `showFailure` (summoned, lvlSpawned, actorFreq, freq, lid, time, mpos)
       return Nothing
-    Just (itemKnownRaw, (itemFullRaw, kit)) -> do
-      (fid, _) <- rndToAction $ oneOf $
-                    possibleActorFactions (itemKind itemFullRaw) factionD
-      pers <- getsServer sperFid
-      let allPers = ES.unions $ map (totalVisible . (EM.! lid))
-                    $ EM.elems $ EM.delete fid pers  -- expensive :(
-          -- Checking skill would be more accurate, but skills can be
-          -- inside organs, equipment, condition organs, created organs, etc.
-          freqNames = map fst $ IK.ifreq $ itemKind itemFullRaw
-          mobile = "mobile" `elem` freqNames
-          aquatic = "aquatic" `elem` freqNames
-      mrolledPos <- case mpos of
-        Just{} -> return mpos
-        Nothing -> do
-          rollPos <-
-            getsState $ rollSpawnPos cops allPers mobile aquatic lid lvl fid
-          rndToAction rollPos
-      case mrolledPos of
-        Just pos ->
-          Just <$> registerActor summoned itemKnownRaw (itemFullRaw, kit)
-                                 fid pos lid time
-        Nothing -> do
-          debugPossiblyPrint
-            "Server: addAnyActor: failed to find any free position"
-          return Nothing
+    NewItem itemGroup itemKnownRaw itemFullRaw itemQuant -> do
+      (fid, _) <- rndToAction $ frequency $
+                    possibleActorFactions [itemGroup] (itemKind itemFullRaw)
+                                          factionD
+      let fact = factionD EM.! fid
+      if isJust $ gquit fact
+      then return Nothing  -- the faction that spawns the monster is dead
+      else do
+        pers <- getsServer sperFid
+        let allPers = ES.unions $ map (totalVisible . (EM.! lid))
+                      $ EM.elems $ EM.delete fid pers  -- expensive :(
+            -- Checking skill would be more accurate, but skills can be
+            -- inside organs, equipment, condition organs, created organs, etc.
+            freqNames = map fst $ IK.ifreq $ itemKind itemFullRaw
+            mobile = IK.MOBILE `elem` freqNames
+            aquatic = IK.AQUATIC `elem` freqNames
+        mrolledPos <- case mpos of
+          Just{} -> return mpos
+          Nothing -> do
+            rollPos <-
+              getsState $ rollSpawnPos cops allPers mobile aquatic lid lvl fid
+            rndToAction rollPos
+        case mrolledPos of
+          Just pos ->
+            Just . (\aid -> (aid, pos))
+            <$> registerActor summoned itemKnownRaw (itemFullRaw, itemQuant)
+                              fid pos lid time
+          Nothing -> do
+            debugPossiblyPrint
+              "Server: addAnyActor: failed to find any free position"
+            return Nothing
+
+addManyActors :: MonadServerAtomic m
+              => Bool -> Int -> Freqs ItemKind -> LevelId -> Time -> Maybe Point
+              -> Int
+              -> m Bool
+addManyActors summoned lvlSpawned actorFreq lid time mpos
+              howMany = assert (howMany >= 1) $ do
+  mInitialLAidPos <- case mpos of
+    Just pos -> return $ Just ([], pos)
+    Nothing ->
+      (\(aid, pos) -> ([aid], pos))
+      <$$> addAnyActor summoned lvlSpawned actorFreq lid time Nothing
+  case mInitialLAidPos of
+    Nothing -> return False  -- suspect content; server debug elsewhere
+    Just (laid, pos) -> do
+      cops@COps{coTileSpeedup} <- getsState scops
+      lvl <- getLevel lid
+      let validTile t = not $ Tile.isNoActor coTileSpeedup t
+          ps = nearbyFreePoints cops lvl validTile pos
+          psNeeded = take (howMany - length laid) ps
+      when (length psNeeded < howMany - length laid) $
+        debugPossiblyPrint $
+          "Server: addManyActors: failed to find enough free positions at"
+          <+> tshow (lid, pos)
+      maidposs <- forM psNeeded $
+        addAnyActor summoned lvlSpawned actorFreq lid time . Just
+      case laid ++ map fst (catMaybes maidposs) of
+        [] -> return False
+        aid : _ -> do
+          b <- getsState $ getActorBody aid
+          mleader <- getsState $ gleader . (EM.! bfid b) . sfactionD
+          when (isNothing mleader) $ setFreshLeader (bfid b) aid
+          return True
 
 rollSpawnPos :: COps -> ES.EnumSet Point
              -> Bool -> Bool -> LevelId -> Level -> FactionId -> State
@@ -130,11 +180,20 @@ rollSpawnPos :: COps -> ES.EnumSet Point
 rollSpawnPos COps{coTileSpeedup} visible
              mobile aquatic lid lvl@Level{larea} fid s = do
   let inhabitants = foeRegularList fid lid s
-      nearInh !df !p = all (\ !b -> df $ chessDist (bpos b) p) inhabitants
+      nearInh !d !p = any (\ !b -> chessDist (bpos b) p < d) inhabitants
+      farInh !d !p = all (\ !b -> chessDist (bpos b) p > d) inhabitants
+      (_, xspan, yspan) = spanArea larea
+      averageSpan = (xspan + yspan) `div` 2
       distantMiddle !d !p = chessDist p (middlePoint larea) < d
+      -- Don't spawn very far from foes, to keep the player entertained,
+      -- but not too close, so that standing on positions with better
+      -- visibility does not influence the spawn places too often,
+      -- to avoid unnatural position micromanagement using AI predictability.
       condList | mobile =
-        [ nearInh (<= 50)  -- don't spawn very far from foes
-        , nearInh (<= 100)
+        [ \p -> nearInh (max 15 $ averageSpan `div` 2) p
+                && farInh 10 p
+        , \p -> nearInh (max 15 $ 2 * averageSpan `div` 3) p
+                && farInh 5 p
         ]
                | otherwise =
         [ distantMiddle 8
@@ -152,12 +211,12 @@ rollSpawnPos COps{coTileSpeedup} visible
                && not (occupiedBigLvl p lvl)
                && not (occupiedProjLvl p lvl) )
     (map (\f p _ -> f p) condList)
-    (\ !p t -> nearInh (> 4) p  -- otherwise actors in dark rooms swarmed
+    (\ !p t -> farInh 3 p  -- otherwise actors in dark rooms swarmed
                && not (p `ES.member` visible)  -- visibility and plausibility
                && (not aquatic || Tile.isAquatic coTileSpeedup t))
-    [ \ !p _ -> nearInh (> 3) p
+    [ \ !p _ -> farInh 3 p
                 && not (p `ES.member` visible)
-    , \ !p _ -> nearInh (> 2) p  -- otherwise actors hit on entering level
+    , \ !p _ -> farInh 2 p  -- otherwise actors hit on entering level
                 && not (p `ES.member` visible)
     , \ !p _ -> not (p `ES.member` visible)
     ]
@@ -230,9 +289,11 @@ swapTime source target = do
   slvl <- getsState $ getLocalTime (blid sb)
   tlvl <- getsState $ getLocalTime (blid tb)
   btime_sb <-
-    getsServer $ (EM.! source) . (EM.! blid sb) . (EM.! bfid sb) . sactorTime
+    getsServer
+    $ fromJust . lookupActorTime (bfid sb) (blid sb) source . sactorTime
   btime_tb <-
-    getsServer $ (EM.! target) . (EM.! blid tb) . (EM.! bfid tb) . sactorTime
+    getsServer
+    $ fromJust . lookupActorTime (bfid tb) (blid tb) target . sactorTime
   let lvlDelta = slvl `timeDeltaToFrom` tlvl
       bDelta = btime_sb `timeDeltaToFrom` btime_tb
       sdelta = timeDeltaSubtract lvlDelta bDelta
@@ -248,9 +309,11 @@ swapTime source target = do
                        `blame` ( slvl, tlvl, btime_sb, btime_tb
                                , sdelta, sdelta', tdelta, tdelta' )) ()
   when (sdelta /= Delta timeZero) $ modifyServer $ \ser ->
-    ser {sactorTime = ageActor (bfid sb) (blid sb) source sdelta $ sactorTime ser}
+    ser {sactorTime = ageActor (bfid sb) (blid sb) source sdelta
+                      $ sactorTime ser}
   when (tdelta /= Delta timeZero) $ modifyServer $ \ser ->
-    ser {sactorTime = ageActor (bfid tb) (blid tb) target tdelta $ sactorTime ser}
+    ser {sactorTime = ageActor (bfid tb) (blid tb) target tdelta
+                      $ sactorTime ser}
 
 updateCalm :: MonadServerAtomic m => ActorId -> Int64 -> m ()
 updateCalm target deltaCalm = do
@@ -271,13 +334,20 @@ updateCalm target deltaCalm = do
 leadLevelSwitch :: MonadServerAtomic m => m ()
 leadLevelSwitch = do
   COps{cocave} <- getsState scops
-  let canSwitch fact = fst (autoDungeonLevel fact)
-                       -- a hack to help AI, until AI client can switch levels
-                       || case fleaderMode (gplayer fact) of
-                            LeaderNull -> False
-                            LeaderAI _ -> True
-                            LeaderUI _ -> False
-      flipFaction (_, fact) | not $ canSwitch fact = return ()
+  factionD <- getsState sfactionD
+  -- Leader switching between levels can be done by the client
+  -- (e.g,. UI client of the human) or by the server
+  -- (the frequency of leader level switching done by the server
+  -- is controlled by @RuleKind.rleadLevelClips@). Regardless, the server
+  -- alwayw does a subset of the switching, e.g., when the old leader dies
+  -- and no other actor of the faction resides on his level.
+  -- Here we check if the server is permitted to handle the mundane cases.
+  let serverMaySwitch fact =
+        bannedPointmanSwitchBetweenLevels fact
+          -- client banned from switching, so the sever has to step in
+        || gunderAI fact
+             -- a hack to help AI, until AI client can switch levels
+      flipFaction (_, fact) | not $ serverMaySwitch fact = return ()
       flipFaction (fid, fact) =
         case gleader fact of
           Nothing -> return ()
@@ -286,7 +356,7 @@ leadLevelSwitch = do
             let !_A = assert (fid == bfid body) ()
             s <- getsServer $ (EM.! fid) . sclientStates
             let leaderStuck = actorWaits body
-                oursRaw =
+                lvlsRaw =
                   [ ((lid, lvl), (allSeen, as))
                   | (lid, lvl) <- EM.assocs $ sdungeon s
                   , lid /= blid body || not leaderStuck
@@ -303,9 +373,9 @@ leadLevelSwitch = do
                   , let allSeen =
                           lexpl lvl <= lseen lvl
                           || CK.cactorCoeff (okind cocave $ lkind lvl) > 150
-                             && not (fhasGender $ gplayer fact)
+                             && not (fhasGender $ gkind fact)
                   ]
-                (oursSeen, oursNotSeen) = partition (fst . snd) oursRaw
+                (lvlsSeen, lvlsNotSeen) = partition (fst . snd) lvlsRaw
                 -- Monster AI changes leadership mostly to move from level
                 -- to level and, in particular, to quickly bring troops
                 -- to the frontline level and so prevent human from killing
@@ -319,8 +389,8 @@ leadLevelSwitch = do
                 -- and sometimes the level with the boss is counted among
                 -- them, but it never happens in the crucial periods when
                 -- AI armies are transferred from level to level.
-                f ((lid, _), _) = abs $ fromEnum lid
-                ours = oursSeen ++ take 2 (sortBy (comparing f) oursNotSeen)
+                f ((_, lvl), _) = ldepth lvl
+                lvls = lvlsSeen ++ take 2 (sortBy (comparing f) lvlsNotSeen)
             -- Actors on desolate levels (not many own or enemy non-projectiles)
             -- tend to become (or stay) leaders so that they can join the main
             -- force where it matters ASAP. Unfortunately, this keeps hero
@@ -328,14 +398,94 @@ leadLevelSwitch = do
             -- so they give back leadership rather quickly to let others follow.
             -- We count non-mobile and sleeping actors, because they may
             -- be dangerous, especially if adjacent to stairs.
-            let freqList = [ (k, (lid, aid))
-                           | ((lid, lvl), (_, (aid, _) : _)) <- ours
+            let overOwnStash b = Just (blid b, bpos b) == gstash fact
+                freqList = [ (k, (lid, aid))
+                           | ((lid, lvl), (_, (aid, b) : rest)) <- lvls
                            , let len = min 20 (EM.size $ lbig lvl)
-                                 k = 1000000 `div` (1 + len) ]
-            unless (null freqList) $ do
+                                 n = 1000000 `div` (1 + len)
+                                 -- Visit the stash guard rarely, but not too
+                                 -- rarely, to regen Calm and fling at foes.
+                                 k = max 1 $ if null rest && overOwnStash b
+                                             then n `div` 30
+                                             else n
+                           ]
+                closeToFactStash (fid2, fact2) = case gstash fact2 of
+                  Just (lid, pos) ->
+                    (fid == fid2 || isFoe fid (factionD EM.! fid) fid2)
+                    && lid == blid body
+                    && chessDist pos (bpos body) == 1  -- visible
+                  Nothing -> False
+                closeToEnemyStash = any closeToFactStash $ EM.assocs factionD
+            foes <- getsState $ foeRegularList fid (blid body)
+            ours <- getsState $ map snd
+                                <$> fidActorRegularAssocs fid (blid body)
+            let foesClose = filter (\b -> chessDist (bpos body) (bpos b) <= 2)
+                                   foes
+                oursCloseMelee =
+                  filter (\b -> chessDist (bpos body) (bpos b) <= 2
+                                && bweapon b - bweapBenign b > 0)
+                         ours
+                canHelpMelee =
+                  not leaderStuck
+                  && length oursCloseMelee >= 2
+                  && not (null foesClose)
+                  && not (all (\b -> any (adjacent (bpos b) . bpos) foes)
+                              oursCloseMelee)
+            unless (closeToEnemyStash || canHelpMelee || null freqList) $ do
               (lid, a) <- rndToAction $ frequency
                                       $ toFreq "leadLevel" freqList
               unless (lid == blid body) $  -- flip levels rather than actors
                 setFreshLeader fid a
-  factionD <- getsState sfactionD
   mapM_ flipFaction $ EM.assocs factionD
+
+-- | Continue or exit or restart the game.
+endOrLoop :: (MonadServerAtomic m, MonadServerComm m)
+          => m () -> (Maybe (GroupName ModeKind) -> m ())
+          -> m ()
+{-# INLINE endOrLoop #-}
+endOrLoop loop restart = do
+  factionD <- getsState sfactionD
+  let inGame fact = case gquit fact of
+        Nothing -> True
+        Just Status{stOutcome=Camping} -> True
+        _ -> False
+      gameOver = not $ any inGame $ EM.elems factionD
+  let getQuitter fact = case gquit fact of
+        Just Status{stOutcome=Restart, stNewGame} -> stNewGame
+        _ -> Nothing
+      quitters = mapMaybe getQuitter $ EM.elems factionD
+      restartNeeded = gameOver || not (null quitters)
+  let isCamper fact = case gquit fact of
+        Just Status{stOutcome=Camping} -> True
+        _ -> False
+      campers = filter (isCamper . snd) $ EM.assocs factionD
+  -- Wipe out the quit flag for the savegame files.
+  mapM_ (\(fid, fact) ->
+    execUpdAtomic $ UpdQuitFaction fid (gquit fact) Nothing Nothing) campers
+  swriteSave <- getsServer swriteSave
+  sstopAfterGameOver <-
+    getsServer $ sstopAfterGameOver . soptions
+  when swriteSave $ do
+    modifyServer $ \ser -> ser {swriteSave = False}
+    writeSaveAll True False
+  if | gameOver && sstopAfterGameOver -> gameExit
+     | restartNeeded -> restart (listToMaybe quitters)
+     | not $ null campers -> gameExit  -- and @loop@ is not called
+     | otherwise -> loop  -- continue current game
+
+gameExit :: (MonadServerAtomic m, MonadServerComm m) => m ()
+gameExit = do
+--  debugPossiblyPrint "Server: Verifying all perceptions."
+  -- Verify that the possibly not saved caches are equal to future
+  -- reconstructed. Otherwise, save/restore would change game state.
+  -- This is done even in released binaries, because it only prolongs
+  -- game shutdown a bit. The same checks at each periodic game save
+  -- would icrease the game saving lag, so they are normally avoided.
+  verifyCaches
+  -- Kill all clients, including those that did not take part
+  -- in the current game.
+  -- Clients exit not now, but after they print all ending screens.
+--  debugPossiblyPrint "Server: Killing all clients."
+  killAllClients
+--  debugPossiblyPrint "Server: All clients killed."
+  return ()
